/* -*- Mode: C++ -*-
 * $Id: Display.cc,v 1.2 2001/04/11 16:07:48 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Display.cc
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.2 $
 * Created by: Johan Thapper 1999-xx-xx
 * Last modified by $Author: andbo928 $ $Date: 2001/04/11 16:07:48 $
 *
 *
 * HISTORY
 *
 * $Log: Display.cc,v $
 * Revision 1.2  2001/04/11 16:07:48  andbo928
 * Fixed display to show a lot of thingys...
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.4  2000/10/19 00:05:34  frehe
 * Made minor changes.
 *
 * Revision 1.3  2000/10/18 23:07:11  frehe
 * Added a say message and a display of them.
 *
 * Revision 1.2  2000/10/11 18:40:29  frehe
 * Upgraded FCFoo.
 *
 * Revision 1.1  2000/09/05 21:12:54  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 */

/**
 * @file Display.cc
 * Implementation file for the class Display.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.2 $
 * $Author: andbo928 $
 * $Date: 2001/04/11 16:07:48 $
 */


#include "TVector.h"
#include "BallState.h"
#include "WorldState.h"
#include "PlayerState.h"
#include "Display.h"
#include "view_constants.h"


RS_BEGIN_NAMESPACE


Display::Display(UInt x, UInt y)
  : viewManager(ViewManager::instance()),
    parameterView(NULL),
    gameView(NULL),
    markerView(NULL),
    agentView(NULL),
    ballView(NULL),
    playerView(NULL),
#ifdef RS_USE_COACH_DEBUGGING
    coachView(NULL),
#endif
    col(x),
    row(y)
{
    parameterView = static_cast<ParameterView*>(viewManager->getView(PARAMETER_VIEW_ID));
    if ( parameterView == NULL ) {
        parameterView = new ParameterView(PARAMETER_VIEW_ID);
        viewManager->addView(parameterView, true);
    }
    gameView = static_cast<GameView*>(viewManager->getView(GAME_VIEW_ID));
    if ( gameView == NULL ) {
        gameView = new GameView(GAME_VIEW_ID);
        viewManager->addView(gameView, true);
    }
    markerView = static_cast<MarkerView*>(viewManager->getView(MARKER_VIEW_ID));
    if ( markerView == NULL ) {
        markerView = new MarkerView(MARKER_VIEW_ID, parameterView);
        viewManager->addView(markerView, true);
    }
    agentView = static_cast<AgentView*>(viewManager->getView(AGENT_VIEW_ID));
    if ( agentView == NULL ) {
        agentView = new AgentView(AGENT_VIEW_ID, parameterView);
        viewManager->addView(agentView, true);
    }
    ballView = static_cast<BallView*>(viewManager->getView(BALL_VIEW_ID));
    if ( ballView == NULL ) {
        ballView = new BallView(BALL_VIEW_ID);
        viewManager->addView(ballView, true);
    }
    playerView = static_cast<PlayerView*>(viewManager->getView(PLAYER_VIEW_ID));
    if ( playerView == NULL ) {
        playerView = new PlayerView(PLAYER_VIEW_ID);
        viewManager->addView(playerView, true);
    }
#ifdef RS_USE_COACH_DEBUGGING
    getView(coachView, COACH_VIEW_ID);
#endif


    
    file.open("/tmp/rs_monitor.txt", ios::out);
    if(!file.is_open())
      cout << "Can't open file '/tmp/rs_monitor.txt' for writing!" << endl;
}


Display::~Display()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(GAME_VIEW_ID);
  viewManager->releaseView(MARKER_VIEW_ID);
  viewManager->releaseView(AGENT_VIEW_ID);
  viewManager->releaseView(BALL_VIEW_ID);
  viewManager->releaseView(PLAYER_VIEW_ID);
#ifdef RS_USE_COACH_DEBUGGING
  viewManager->releaseView(COACH_VIEW_ID);
#endif
}


void Display::setLastSkill(const std::string& last)
{
    lastSkill = last;
    cout << "lastSkill called." << endl;
}


void Display::addArrow(const TVector& start, 
		       const TVector& end, 
		       double duration,
		       string color)
{
  arrowStart.insert(arrowStart.end(), start);
  arrowEnd.insert(arrowEnd.end(), end);
  arrowDuration.insert(arrowDuration.end(), duration);
  arrowColor.insert(arrowColor.end(), color);
}


void Display::addMessage(string msg)
{
  messages.insert(messages.end(), msg);
}

void Display::draw()
{

  if(file.is_open())
    {
      // Create a testworldstate.
      WorldState ws;
      {
	TVector pos(0,0,1);
	TVector delta(5,0,1);
	vector<PlayerState> oppVec;
	for(int i = 0; i < 5; i++)
	  {
	    PlayerState plr;
	    plr.setPosition(pos);
	    plr.setSpeed(TVector(1,0,0));
	    plr.setDirection(0);
	    plr.setUniformNumber(i+1);
	    oppVec.insert(oppVec.end(), plr);
	    pos = pos + delta;
	  }
	ws.setOpponents(oppVec);

	vector<PlayerState> tmVec;
	pos = TVector(-5, 0, 1);
	delta = TVector(-5, 0, 1);
	for(int i = 0; i < 6; i++)
	  {
	    PlayerState plr;
	    plr.setPosition(pos);
	    plr.setSpeed(TVector(1,0,1));
	    plr.setDirection(0);
	    plr.setUniformNumber(i+1);
	    oppVec.insert(oppVec.end(), plr);
	    pos = pos + delta;
	  }
	ws.setTeammates(oppVec);

	AgentState ag;
	ag.setPosition(TVector(20,20,1));
	ag.setSpeed(TVector(0,1,1));
	ag.setDirection(0);
	ag.setUniformNumber(11);
	ag.setStamina(1300);
	ws.setAgent(ag);
	
	BallState ball;
	TVector tst = TVector(3,3,1);
	tst.setX(3);
	ball.setPosition(TVector(3,3,1));
	ball.setSpeed(TVector(1,0,1));
	ws.setBall(ball);

	addArrow(TVector(0,0), TVector(10,10));
	addArrow(TVector(0,0), TVector(-10,10), 1, "magenta");
      }

      // Draw a worldstate

      file << "(cycle ";
      file << "(field ";
      
      // Draw the ball.
      TVector ballPos = ws.getBall().getPosition();
      file << "(ball " 
	   << ballPos.getX() << " "
	   << ballPos.getY() << " "
	   << ballPos.getC() << ") ";
      
      // Draw the agent.
      TVector agentPos = ws.getAgent().getPosition();
      double agentDir = ws.getAgent().getDirection();
      int agentNum = ws.getAgent().getUniformNumber();
      file << "(Player "
	   << agentPos.getX() << " "
	   << agentPos.getY() << " "
	   << "our "
	   << agentNum << " "
	   << agentDir << " "
	   << agentPos.getC() << ") ";
      
      // Draw the teammates.
      const vector<PlayerState>& tmVec = ws.getTeammates();
      for(unsigned int i = 0; i < tmVec.size(); i++)
	{
	  TVector plrPos = tmVec[i].getPosition();
	  double plrDir = tmVec[i].getDirection();
	  int plrNum = tmVec[i].getUniformNumber();
	  file << "(player "
	       << plrPos.getX() << " "
	       << plrPos.getY() << " "
	       << "our "
	       << plrNum << " "
	       << plrDir << " "
	       << plrPos.getC() << ") ";
	}
      

      // Draw the opponents.
      const vector<PlayerState>& oppVec = ws.getOpponents();
      for(unsigned int i = 0; i < oppVec.size(); i++)
	{
	  TVector plrPos = oppVec[i].getPosition();
	  double plrDir = oppVec[i].getDirection();
	  int plrNum = oppVec[i].getUniformNumber();
	  file << "(player "
	       << plrPos.getX() << " "
	       << plrPos.getY() << " "
	       << "their "
	       << plrNum << " "
	       << plrDir << " "
	       << plrPos.getC() << ") ";
	}
      
      // Draw arrows if any.

      for(unsigned int i = 0; i < arrowStart.size(); i++)
	{
	  file << "(arrow "
	       << arrowStart[i].getX() << " "
	       << arrowStart[i].getY() << " "
	       << arrowEnd[i].getX() << " "
	       << arrowEnd[i].getY() << " "
	       << arrowDuration[i] << " "
	       << arrowColor[i] << ")";
	}
      arrowStart.clear();
      arrowEnd.clear();
      arrowDuration.clear();
      arrowColor.clear();

      file << ")"; /* field */



      file << "(status "
	   << ws.getAgent().getUniformNumber() << " "
	   << ws.getAgent().getStamina() << ")";

      // Print messages.

      
      for(unsigned int i = 0; i < messages.size(); i++)
	{
	  file << "(message \""
	       << messages[i] << "\")";
	}
      messages.clear();

      file << ")";
    }
}


RS_END_NAMESPACE
