/* -*- Modex: C++ -*-
 * $Id: InPenaltyArea.cc,v 1.5 2001/07/28 07:17:57 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: InPenaltyArea.cc
 * Unitname: Framework/predicate
 * $Revision: 1.5 $
 * Created by: Bjrn, Ola och Jenny 2000-09-12
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 07:17:57 $
 *
 *
 * HISTORY
 *
 */

/**
 * @file InPenaltyArea.cc
 * Implementation file for the predicate class InPenaltyArea.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.5 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:17:57 $
 */


#include "InPenaltyArea.h"
#include "view_constants.h"
#include "WorldStateView.h"


RS_BEGIN_NAMESPACE


InPenaltyArea::InPenaltyArea()
  : Predicate(),
    worldStateView(NULL)
{
  getView(worldStateView, WORLD_STATE_VIEW_ID);
}

InPenaltyArea::InPenaltyArea(const InPenaltyArea& obj)
  : Predicate(obj),
    worldStateView(obj.worldStateView)
{
}


InPenaltyArea::~InPenaltyArea()
{
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}


InPenaltyArea& InPenaltyArea::operator= (const InPenaltyArea& obj)
{
  if ( this != &obj ) {
    Predicate::operator=(obj);
    worldStateView = obj.worldStateView;
  }
  
  return *this;
}


FuzzyBool InPenaltyArea::evaluate()
{
//    const double minX = -52.5;
  const double maxX = -36;
  const double minY = -20.16;
  const double maxY = 20.16;

  const double buffer = 1.0;
  
  TVector pos = worldStateView->getWorldState().getAgent().getPosition();

  double x = maxX - pos.getX();
  double y1 = maxY - pos.getY();
  double y2 = pos.getY() - minY;

  double minBuffer = min(x, min(y1, y2));

  return FuzzyBool(minMax(0.0,minBuffer/buffer + 0.5 ,1.0));
}


void InPenaltyArea::update(const UInt32)
{
}

RS_END_NAMESPACE
