/* -*- Mode: C++ -*-
 * $Id: WeHaveBall.h,v 1.2 2001/07/28 07:17:57 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: WeHaveBall.h
 * Unitname: Framework/predicate
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-20
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 07:17:57 $
 *
 *
 * HISTORY
 *
 * $Log: WeHaveBall.h,v $
 * Revision 1.2  2001/07/28 07:17:57  mikbr212
 * Removed warnings!!
 *
 * Revision 1.1  2001/04/17 14:55:27  andbo928
 * Created... might be buggy...
 *
 * Revision 1.2  2001/04/17 12:47:59  andbo928
 * It now uses worldstateview.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.1  2000/09/20 14:42:48  frehe
 * Added the predicate WeHaveBall that will return true iff the agent is the closest player to the ball.
 *
 */

/**
 * @file WeHaveBall.h
 * Include file for the predicate class WeHaveBall.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:17:57 $
 */


#ifndef WEHAVEBALL_H
#define WEHAVEBALL_H

#include "Predicate.h"
#include "WorldStateView.h"

RS_BEGIN_NAMESPACE

/**
 * @class WeHaveBall WeHaveBall.h
 * This predicate returns @c true iff the agent is the closest player
 * to the ball.
 *
 * @par Unit:
 *      Framework/predicate
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:17:57 $
 */
class WeHaveBall : public Predicate {
public:
  /** Create a predicate with default behavior. */
  WeHaveBall();

  /** Creates a predicate that is an exact copy of @a obj. */
  WeHaveBall(const WeHaveBall& obj);

  /** Destroys the predicate. */  
  virtual ~WeHaveBall();


  /** Assignment operator. Assigns @a obj to this predicate. */
  WeHaveBall& operator= (const WeHaveBall&);


  /** Function application operator. Will call evaluate(). */
  virtual FuzzyBool operator()();
  virtual FuzzyBool operator()(const WorldState& ws);


  /** Will return @c true iff the agent is the closest player to the ball. */
  virtual FuzzyBool evaluate();
  virtual FuzzyBool evaluate(const WorldState& ws);

  
protected:
  /** A pointer to the ball view. */
  WorldStateView* worldView;

  /** The update function, should be used to update the value of the
    predicate if necessary. The method needUpdate(const UInt32 update_number)
    can be used to check if the predicate needs to be updated based on the
    last update number it was updated for. */
  virtual void update(const UInt32 update_number);

  // Help function.
  double WeHaveBall::getDistToPlayers(const vector<PlayerState>& players,
				      TVector point);


};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "WeHaveBall.icc"
#endif


#endif // !WEHAVEBALL_H
