/* -*- Mode: C++ -*-
 * $Id: CoverSpan.cc,v 1.2 2001/07/28 07:18:12 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * Filename: CoverSpan.h
 * Unitname: Framework/view
 * $Revision: 1.2 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 07:18:12 $
 *
 *
 * HISTORY
 *
 * $Log: CoverSpan.cc,v $
 * Revision 1.2  2001/07/28 07:18:12  mikbr212
 * Removed warnings!!
 *
 * Revision 1.1  2001/07/12 12:56:20  andbo928
 * Added CoverSpan.
 *
 * Revision 1.8  2001/04/24 21:39:57  mikbr212
 * added avoid-offside
 *
 * Revision 1.7  2001/04/24 21:25:33  mikbr212
 * fixed bug
 *
 * Revision 1.6  2001/04/24 20:50:50  mikbr212
 * ??
 *
 * Revision 1.5  2001/04/24 19:12:28  mikbr212
 * added sticky
 *
 * Revision 1.4  2001/04/24 14:10:12  sigge
 * Mark players 3 meters ahead of them.
 *
 * Revision 1.3  2001/04/24 12:30:47  mikbr212
 * fixed
 *
 * Revision 1.2  2001/04/23 22:57:11  mikbr212
 * Killed and made a new skill. Not done yet
 *
 *
 */

/**
 * @file CoverSpan.cc
 * Implementation of the CoverSpan skill.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.2 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:18:12 $
 */

#include "CoverSpan.h"
#include <algorithm>
#include "Display.h"

RS_BEGIN_NAMESPACE

// --- CONSTRUCTORS ---

/**
 * Constructor.
 */

CoverSpan::CoverSpan()
  : MarkPosition(),
    worldStateView(NULL),
    parameterView(NULL)
{
  getView(worldStateView, WORLD_STATE_VIEW_ID);
  getView(parameterView, PARAMETER_VIEW_ID);
}

// --- DESTRUCTOR ---

CoverSpan::~CoverSpan()
{
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
  viewManager->releaseView(PARAMETER_VIEW_ID);
}

// --- METHODS ---
/** Calculates which spans are currently uncoverd.
 */
void CoverSpan::insertSpan(Span s, list<CoverSpan::Span>& spans)
{
  if(s.start < 1.0 && s.end > 0.0)
    {
      list<CoverSpan::Span>::iterator spanIt;
      for(spanIt = spans.begin(); spanIt != spans.end(); spanIt++)
      {
	// Check if we're done.
	if(s.start > spanIt->end)
	{
	  break; // We're done.
	}
	else
	{
	  if(s.end > spanIt->start)
	  {
	    if(s.start < spanIt->start
	       && s.end > spanIt->end)
	    {
	      // Remove the old span.
//  	      cout << "Remove span." << endl;
	      spanIt = spans.erase(spanIt);
	    }
	    else if(s.start < spanIt->start)
	    {
	      // Cut of start piece.
//  	      cout << "Trim start." << endl;
	      spanIt->start = s.end;
	    }
	    else if(s.end > spanIt->end)
	    {
	      // Cut of end piece;
//  	      cout << "Trim end." << endl;
	      spanIt->end = s.start;
	    }
	    else
	    {
	      // Split span into subspans.
//  	      cout << "Split span." << endl;
	      CoverSpan::Span newSpan;
	      newSpan.start = spanIt->start;
	      newSpan.end = s.start;
	      spanIt->start = s.end;
	      spanIt = spans.insert(spanIt, newSpan);
	    }
	  }
	}
      }
    }
}

void CoverSpan::calculateSpans(TVector threat, 
				     TVector& spanStart,
				     TVector& spanEnd)
{
  TLine goalLine = TLine(TVector(-52.5, -parameterView->SP_goal_width/2.0),
			 TVector(-52.5, parameterView->SP_goal_width/2.0));

  list<CoverSpan::Span> spans;
  CoverSpan::Span init;
  init.start = 0.0;
  init.end = 1.0;
  spans.push_back(init);

  const vector<PlayerState>& tm =
    worldStateView->getWorldState().getTeammates();

  // Assume that the agent is the goalie, for now...
  vector<PlayerState>::const_iterator it;
  for(it = tm.begin(); it != tm.end(); it++)
  {
    // Magic computation of the point where the extension
    // of the threat point touches the players kickable-area.
    TVector tToP = it->getPosition() - threat;
    double actionRadius = parameterView->SP_kickable_area;

    if(tToP.getLength() < actionRadius)
    {
      // Someone has the ball within his actionradius, the goal is covered.
      spans.clear();
      break;
    }

    double alfa = std::acos(actionRadius / tToP.getLength()) * 180.0 / M_PI;
    TVector delta1 = -(tToP.normalized());
    delta1 *= actionRadius;
    TVector delta2 = delta1;
    delta1.rotate(-alfa);
    delta2.rotate(alfa);

//      cout << "alfa: " << alfa;

    TLine line1 = TLine(threat, threat + tToP + delta1);
    TLine line2 = TLine(threat, threat + tToP + delta2);

    double s1, tStart;
    goalLine.getIntersection(line1, tStart, s1);
    double s2, tEnd;
    goalLine.getIntersection(line2, tEnd, s2);
    CoverSpan::Span s;
    s.start = tStart;
    s.end = tEnd;

//      cout << ", tStart: " << tStart;
//      cout << ", tEnd: " << tEnd << ", s: " << s1 << ", " << s2 << endl;
    if(s1 > 0.0 && s2 > 0.0)
    {
      insertSpan(s, spans);
    }
  }

  // Initialize to defalutvalues.
  spanStart = TVector(-52.5, 0);
  spanEnd = TVector(-52.5, 0);

  list<CoverSpan::Span>::iterator spanIt;
  double maxSpan = 0.0;
//    double t;
  for(spanIt = spans.begin(); spanIt != spans.end(); spanIt++)
  {
//      cout << "Cur span: " << spanIt->start << ", " << spanIt->end << endl;
    if( (spanIt->end - spanIt->start) > maxSpan )
    {
      maxSpan = spanIt->end - spanIt->start;
      spanStart = goalLine.getStart() + goalLine.getVector() * spanIt->start;
      spanEnd = goalLine.getStart() + goalLine.getVector() * spanIt->end;
    }
  }
}

Plan CoverSpan::generatePlan() {
  const BallState& ball = worldStateView->getWorldState().getBall();
  const AgentState& agent = worldStateView->getWorldState().getAgent();

  // Calculation of actionradius.
  TVector bToA = agent.getPosition() - ball.getPosition();
  double timeToAgent = 
    floor(bToA.getLength() / parameterView->SP_ball_speed_max);

  //double actionRadius = 0.8 * parameterView->SP_catchable_area_l;
  double actionRadius = 0.8 * parameterView->SP_kickable_area;
  if(timeToAgent > 4)
    actionRadius += (timeToAgent-4) * parameterView->SP_player_speed_max;

  // The interesting parameters.
  TVector focusPoint = ball.getPosition();
  static TVector lastFocusPoint;
  focusPoint = (focusPoint + lastFocusPoint) / 2.0;
  lastFocusPoint = focusPoint;
  TVector spanStart;
  TVector spanEnd;
  calculateSpans(focusPoint, spanStart, spanEnd);

//    cout << "Start: " << spanStart
//         << ", End: " << spanEnd << endl;

  // Start and end vectors of the span to cover.
  TVector edgeVec1 = spanStart - focusPoint;
  TVector edgeVec2 = spanEnd - focusPoint;

  TVector minEdge;
  if(edgeVec1.getLength() < edgeVec2.getLength())
    minEdge = edgeVec1;
  else
    minEdge = edgeVec2;

  // Set the same length to both vectors.
  edgeVec2.setLength(edgeVec1.getLength());
  TVector edgeMid = (edgeVec1 + edgeVec2) / 2.0;

  // Calculate the base of the triangle.
  TVector baseVec = edgeVec2 - edgeVec1;

  // Calculate the sweet spot.
  TVector cutOffSpot = 
    2*actionRadius*edgeMid / baseVec.getLength();
  if(cutOffSpot.getLength() > minEdge.getLength())
    cutOffSpot.setLength(minEdge.getLength());
  cutOffSpot += focusPoint;

  double markDist = (TVector(-52.5, 0) - cutOffSpot).getLength();
  setLookAtMark(false);
  setDistance(markDist);
  setPosition((spanStart + spanEnd) / 2.0);

  return MarkPosition::generatePlan();
}

FuzzyBool CoverSpan::persistent() {
  return false;
}

FuzzyBool CoverSpan::applicable() {
  return MarkPosition::applicable();

}

RS_END_NAMESPACE







