/* -*- Mode: C++ -*-
 * $Id: FindBall.cc,v 1.6 2001/07/26 13:46:38 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FindBall.cc
 * Unitname: Framework/skill
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: andbo928 $ $Date: 2001/07/26 13:46:38 $
 *
 *
 * HISTORY
 * $Log: FindBall.cc,v $
 * Revision 1.6  2001/07/26 13:46:38  andbo928
 * Now uses gaussian stuff to weight the treshold.
 *
 * Revision 1.5  2001/04/24 17:10:00  mikbr212
 * updated a lot
 *
 * Revision 1.4  2001/04/23 15:06:54  sigge
 * Compensation for neck turning.
 *
 * Revision 1.3  2001/04/16 11:22:58  mikbr212
 * did stuff
 *
 * Revision 1.2  2001/04/16 11:11:33  mikbr212
 * Now it seems to work
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.5  2000/10/07 15:07:36  matka134
 * Now using the predicate IsBallKnown
 *
 * Revision 1.4  2000/09/29 17:55:02  olalu526
 * Added FaceDirection, fixed some in FindBall (now works as expected), fixed segmentation fault in TurnDirection and more.
 *
 * Revision 1.3  2000/09/28 18:52:17  olalu526
 * Now the agent works quite fine.
 *
 * Revision 1.2  2000/09/13 01:38:59  frehe
 * Updated the skills to use getView instead of static_cast<...>.
 *
 * Revision 1.1  2000/09/05 21:12:01  frehe
 * #include rs_log.h
 *
 */

/**
 * @file FindBall.cc
 * Implementation file for the skill class FindBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.6 $
 * $Author: andbo928 $
 * $Date: 2001/07/26 13:46:38 $
 */


#include "FindBall.h"

RS_BEGIN_NAMESPACE


FindBall::FindBall(Float cThr)
  : Skill(),
    parameterView(NULL),
    worldStateView(NULL),
    confThr(cThr)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(worldStateView, WORLD_STATE_VIEW_ID);
} 


FindBall::~FindBall()
{
  viewManager->releaseView(PARAMETER_VIEW_ID);
  viewManager->releaseView(WORLD_STATE_VIEW_ID);
}


void FindBall::setThreshold(Float cThr) {
  confThr = cThr;
}

FuzzyBool FindBall::applicable(Float cThr) {
  setThreshold(cThr);
  return applicable();
}
  
Plan FindBall::generatePlan()
{
  static int lookAtBallCycle = 0;
  static double turnAngle = 40;

  const AgentState& agent = worldStateView->getWorldState().getAgent();
  const BallState& ball = worldStateView->getWorldState().getBall();

  TVector relBallPos(ball.getPosition() - agent.getPosition());
  relBallPos.rotate(-agent.getDirection());
  
  AngleDeg faceRelBallDir(relBallPos.getAngle() - agent.getNeckDirection());

  // Has agent looked at ball since ball was lost?
  if(abs(faceRelBallDir.getDeg()) > 20 && 
     worldStateView->getLastSeeCycle() - lookAtBallCycle > 10) {
    // No. Look at ball (where it should be).
    lookAtBallCycle = worldStateView->getWorldState().getGameTime();
    AgentState newAgent(agent);

    // Can I turn to ball?
    if(newAgent.doActualTurn(relBallPos.getAngle())) {
      newAgent.doTurnNeck(-newAgent.getNeckDirection());
    } else { // No, do a turnNeck also
      newAgent.step();
      double bodyDir = newAgent.getDirection();
      
      newAgent = agent;
      newAgent.doActualTurn(relBallPos.getAngle());
      newAgent.doTurnNeckTo(AngleDeg(relBallPos.getAngle() - 
				     bodyDir - newAgent.getNeckDirection()).getDeg());
    }
    // compute turnAngle
    newAgent.step();
    TVector relVectorToOurGoal(TVector(-52.5, 0) - newAgent.getPosition());
    relVectorToOurGoal.rotate(-newAgent.getDirection());
    if(AngleDeg(relVectorToOurGoal.getAngle()).getDeg() > 0) 
      turnAngle = 40;
    else
      turnAngle = -40;

    return Plan(newAgent.getStep());
  }

  // No idea where ball is, just turn
  AgentState newAgent(agent);
  newAgent.doActualTurn(turnAngle + newAgent.getNeckDirection());
  newAgent.doTurnNeck(-newAgent.getNeckDirection());

  return Plan(newAgent.getStep());
}


FuzzyBool FindBall::persistent()
{
  return false;
}

FuzzyBool FindBall::applicable()
{
  // Require higher confidence close to the player.
  TVector delta =
    worldStateView->getWorldState().getAgent().getPosition()
    - worldStateView->getWorldState().getBall().getPosition();
  
  // Gauss. 0.5 @ 53m
  float weightThr = confThr * exp(-0.0869 * delta*delta);

  return (worldStateView->getWorldState().getBall().getC() < weightThr);
}


RS_END_NAMESPACE
