#ifndef FUZZYTELEPORT_H
#define FUZZYTELEPORT_H


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"

#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "GameView.h"
#include "WorldStateView.h"
#include "rs_log.h"

RS_BEGIN_NAMESPACE

/**
 * @class FuzzyTeleport FuzzyTeleport.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.4 $
 * $Author: sigge $
 * $Date: 2001/04/20 17:59:21 $
 */

class FuzzyTeleport : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  FuzzyTeleport();

  /** Destructor should clean up after the skill. */
  virtual ~FuzzyTeleport();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

protected:
  Point calculateMovePos();

  ParameterView* parameterView;
  MarkerView* markerView;
  GameView* gameView;
  WorldStateView* worldStateView;

  // Last cycle we issued a teleport
  int lastTeleport;
  float clearSafeness;

  /** Hide copy-constructor. */
  FuzzyTeleport(const FuzzyTeleport&);
  /** Hide assignment operator. */
  FuzzyTeleport& operator= (const FuzzyTeleport&);
};


RS_END_NAMESPACE

#endif // !FUZZYCLEARBALL_H
