#ifndef _GENESKILL_H_
#define _GENESKILL_H_


// Include the system header which definitions you use.
//#include <...>
//#include <...>

// Include the header files which definitions you use.
#include "Skill.h"

#include "CoordVector.h"
#include "CoordPoint.h"
#include "view_constants.h"
#include "ParameterView.h"
#include "MarkerView.h"
#include "AgentView.h"
#include "BallView.h"
#include "FuzzyPlayerView.h"
#include "Operation.h"

RS_BEGIN_NAMESPACE

//class ParameterView;
//class MarkerView;
//class AgentView;
//class BallView;

/**
 * @class GeneSkill GeneSkill.h
 * This class implements the pass skill
 *
 * <!--
 * @pre HaveBall, visible players.
 * @post Ball hopefully at the closest passable player.
 * @invariant ?!?!
 *
 * @todo Finetune the skill.
 * -->
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:58 $
 */

class GeneSkill : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  GeneSkill();

  /** Destructor should clean up after the skill. */
  virtual ~GeneSkill();
  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan();

  virtual void setGene(Operation *gn);

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent();

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable();

protected:
  Vector calculateClearVector();
  bool isBallKickable();

  ParameterView* parameterView;
  MarkerView* markerView;
  AgentView* agentView;
  FuzzyBallView* ballView;
  FuzzyPlayerView* playerView;
  KickBallToSpeedVectorMulti kickBall;

  std::string trim(std::string str);
  std::string first(std::string str);
  std::string rest(std::string str);

  Operation *gene;
  float clearSafeness;
  
  /** Hide copy-constructor. */
  GeneSkill(const GeneSkill&);
  /** Hide assignment operator. */
  GeneSkill& operator= (const GeneSkill&);
};


RS_END_NAMESPACE

#endif // !FUZZYCLEARBALL_H
