/* -*- Mode: C++ -*-
 * $Id: GoalieIntention.h,v 1.8 2001/07/09 14:47:33 andbo928 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.8 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: andbo928 $ $Date: 2001/07/09 14:47:33 $
 *
 *
 * HISTORY
 *
 * $Log: GoalieIntention.h,v $
 * Revision 1.8  2001/07/09 14:47:33  andbo928
 * New code, now covers the largest uncoverd span in the goal.
 *
 * Revision 1.7  2001/07/06 14:30:29  sigge
 * Fixed abstraction bug.
 *
 * Revision 1.6  2001/07/06 14:07:11  andbo928
 * Added modelutility
 *
 * Revision 1.5  2001/07/06 14:03:09  andbo928
 * Added modelutility
 *
 * Revision 1.4  2001/07/06 12:38:30  andbo928
 * Zonked applicable.
 *
 * Revision 1.3  2001/07/03 15:24:18  andbo928
 * Added utility methods.
 *
 * Revision 1.2  2001/07/03 13:15:29  sigge
 * GoalieIntention now uses the old goalie behaviour.
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file Intention.h
 * Definition file for the class Intention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.8 $
 * $Author: andbo928 $
 * $Date: 2001/07/09 14:47:33 $
 */

#ifndef GOALIE_INTENTION_H
#define GOALIE_INTENTION_H

#include "Intention.h"
#include <list>

RS_BEGIN_NAMESPACE

/** 
 * @class GoalieIntention GoalieIntention.h
 * This intention is used to model and be a decision maker for
 * the goalies (there are two). 
 *
 * $Revision: 1.8 $
 * $Author: andbo928 $
 * $Date: 2001/07/09 14:47:33 $
 */
class GoalieIntention : public Intention {
public:
  GoalieIntention();
  virtual ~GoalieIntention();

  // Uncomment either chooseSkill or generatePlan
  //  virtual Plan generatePlan();
  virtual void model(WorldState&);

  /** Return @c the utility value of the intention.
   * This should be a light weight method, since this
   * will be called for all other players.
   */
  virtual Utility modelUtility(const WorldState&);

  // Uncomment if the agent should be threated specially
  //virtual Utility utility();
  virtual Utility utility();

  virtual float getPersistence() const;

  virtual IntentionId getID() const;
  
protected:
  struct Span {
    double start;
    double end;
  };

  // Uncomment either chooseSkill or generatePlan
  virtual Skill* chooseSkill();

  void insertSpan(Span s, list<Span>& spans);
  void calculateSpans(TVector threat, 
		      TVector& spanStart,
		      TVector& spanEnd);

  // This method should not need to be overloaded
  //  virtual Plan getPlan(bool clear=false);

  // The following method is inherited:
  //  virtual Plan getPlan(bool clear=false);
  
  // The following views are inherited:
  //    ParameterView* parameterView;
  //    WorldStateView* worldStateView;
  //    PassTreeView* passTreeView;
  //    GameView* gameView;
  
private:
  /** Hide copy-constructor. */
  GoalieIntention(const GoalieIntention&);
  /** Hide assignment operator. */
  GoalieIntention& operator= (const GoalieIntention&);
};

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "GoalieIntention.icc"
#endif

#endif // GOALIE_INTENTION_H
