/* -*- Mode: C++ -*-
 * $Id: KickBallToSpeedVector.h,v 1.5 2001/04/24 14:09:44 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: TurnBall.h
 * Unitname: Framework/skill
 * $Revision: 1.5 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: sigge $ $Date: 2001/04/24 14:09:44 $
 *
 *
 * HISTORY
 * $Log: KickBallToSpeedVector.h,v $
 * Revision 1.5  2001/04/24 14:09:44  sigge
 * Added emergency predicate to decide when to use one kick only.
 *
 * Revision 1.4  2001/04/20 19:01:13  mikbr212
 * added default constructor
 *
 * Revision 1.3  2001/04/14 19:26:08  mikbr212
 * Uses a single plan. Works good.
 *
 * Revision 1.2  2001/04/14 19:10:46  mikbr212
 * Bad skill which refreshes the ballState every cycle
 *
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.4  2000/10/09 18:48:09  mikbr212
 * Added comments
 *
 * Revision 1.3  2000/10/07 14:26:55  mikbr212
 * *** empty log message ***
 *
 * Revision 1.2  2000/09/27 14:29:59  mikbr212
 * fixed ballspeed bug
 *
 * Revision 1.1  2000/09/24 19:30:19  andbo928
 * Various skills.
 *
 * Revision 1.1  2000/09/05 21:12:02  frehe
 * #include rs_log.h
 *
 */

#ifndef KICKBALLTOSPEEDVECTOR_H
#define KICKBALLTOSPEEDVECTOR_H


#include "Skill.h"
#include "TLine.h"
#include "TVector.h"

#include "KickCommand.h"

#include "ParameterView.h"
#include "WorldStateView.h"

#include "rs_log.h"
#include "view_constants.h"



RS_BEGIN_NAMESPACE


/**
 * @class KickBallToSpeedVector KickBallToSpeedVector.h
 * This class encapsulates a kicking skill.
 *
 * 
 * @pre The preconditions for the skill are:
 * 1) ball in kickable_area
 * 2) no obstacle on straight line from current ball position to next ball position 
 * @post The postconditions for the skill are...
 * 
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.5 $
 * $Author: sigge $
 * $Date: 2001/04/24 14:09:44 $
 */
class KickBallToSpeedVector : public Skill {
public:
  KickBallToSpeedVector();
  KickBallToSpeedVector(const TVector& vec);
  virtual ~KickBallToSpeedVector();

  virtual void setVector(const TVector& vec);
  virtual TVector getVector() const;
  
  virtual Plan generatePlan();
  virtual Plan generatePlan(const TVector& vec);

  virtual FuzzyBool persistent();

  virtual FuzzyBool applicable();
  virtual FuzzyBool applicable(const TVector& vec);

protected:
  void computePositions();
  bool emergency();

  std::vector<TVector> positions;
  TVector vector; 
  ParameterView* parameterView;
  WorldStateView* worldStateView;
};


RS_END_NAMESPACE


#endif // !KICKBALLTOSPEEDVECTOR_H


