
//  #include "InGoalArea.h"
//  #include "AgentView.h"
//  #include "MarkerView.h"
//  #include "view_constants.h"
// #include "ParameterView.h"
// #include <stdio.h>
#include "ObjMath.h"

RS_BEGIN_NAMESPACE

/****************************************
 * Operation for extracting agent pos   *
 ****************************************/

OpAgentPos::OpAgentPos() : 
  PointOperation(),
  viewManager(ViewManager::instance()),
  parameterView(NULL),
  agentView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

OpAgentPos::~OpAgentPos()
{
  viewManager->releaseView(parameterView->getId());
  viewManager->releaseView(agentView->getId());
}

std::vector<opType> OpAgentPos::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.clear();
  return aTypes;
}

Point OpAgentPos::getPoint()
{
  if (agentView->getAgentPosition(NOW).isKnown())
    return agentView->getAgentPosition(NOW).getPoint();

  return Point(0,0);
}

std::string OpAgentPos::getOperationString()
{
  return "pagentpos";
}

/*******************************************
 * Operation for extracting agent speed    *
 *******************************************/

OpAgentSpeed::OpAgentSpeed() : 
  PointOperation(),
  viewManager(ViewManager::instance()),
  parameterView(NULL),
  agentView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

OpAgentSpeed::~OpAgentSpeed()
{
  viewManager->releaseView(parameterView->getId());
  viewManager->releaseView(agentView->getId());
}

std::vector<opType> OpAgentSpeed::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.clear();
  return aTypes;
}

Point OpAgentSpeed::getPoint()
{
  if (agentView->getSpeedVector(NOW).isKnown())
    return Point(agentView->getSpeedVector(NOW).getVector());

  return Point(0,0);
}

std::string OpAgentSpeed::getOperationString()
{
  return "pagentspeed";
}

/*******************************************
 * Operation for extracting agent speed    *
 *******************************************/

OpAgentDir::OpAgentDir() : 
  FloatOperation(),
  viewManager(ViewManager::instance()),
  parameterView(NULL),
  agentView(NULL)
{
  getView(parameterView, PARAMETER_VIEW_ID);
  getView(agentView, AGENT_VIEW_ID, parameterView);
}

OpAgentDir::~OpAgentDir()
{
  viewManager->releaseView(parameterView->getId());
  viewManager->releaseView(agentView->getId());
}

std::vector<opType> OpAgentDir::getArgumentTypes()
{
  std::vector<opType> aTypes;
  aTypes.clear();
  return aTypes;
}

float OpAgentDir::getFloat()
{
  if (agentView->getBodyDirection(NOW).isKnown())
    return agentView->getBodyDirection(NOW).getAngle().getAngle().getAngle().getDeg();
  return 0.0;
}

std::string OpAgentDir::getOperationString()
{
  return "fagentdir";
}

RS_END_NAMESPACE









