/* -*- Mode: C++ -*-
 * $Id: ReceiveBallIntention.cc,v 1.12 2001/07/28 07:18:12 mikbr212 Exp $
 *
 * IDENTIFICATION
 *
 * $Revision: 1.12 $
 * Created by: Mikael Brnnstrm, Andreas Bckert, Andreas Sigfridsson
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/28 07:18:12 $
 *
 *
 * HISTORY
 *
 * $Log: ReceiveBallIntention.cc,v $
 * Revision 1.12  2001/07/28 07:18:12  mikbr212
 * Removed warnings!!
 *
 * Revision 1.11  2001/07/09 14:00:35  sigge
 * Moved return statement from evalWorldState to utility (bug) and removed spam.
 *
 * Revision 1.10  2001/07/06 14:34:30  andbo928
 * Fixed modelUtility
 *
 * Revision 1.9  2001/07/06 14:33:15  andbo928
 * Fixed utility and stuff.
 *
 * Revision 1.8  2001/07/06 10:00:46  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.7  2001/07/03 16:25:47  andbo928
 * Implemented the utility method.
 *
 * Revision 1.6  2001/07/03 15:24:19  andbo928
 * Added utility methods.
 *
 * Revision 1.5  2001/07/03 13:24:46  andbo928
 * Now checks ball confidence.
 *
 * Revision 1.4  2001/07/03 12:49:10  andbo928
 * Added 10m radius check in applicable.
 *
 * Revision 1.3  2001/07/02 15:59:54  andbo928
 * New implementation, not fully functional yet.
 *
 * Revision 1.2  2001/07/01 14:16:00  sigge
 * Added Mickes code (Intentions)
 *
 * Revision 1.1  2001/06/19 21:35:10  mikbr212
 * Created stubs
 *
 * Revision 1.1  2001/06/19 14:11:43  mikbr212
 * Created
 *
 */

/**
 * @file ReceiveBallIntention.cc
 * Implementation file for the class ReceiveBallIntention.
 *
 * @par Unit:
 *      Framework
 * $Revision: 1.12 $
 * $Author: mikbr212 $
 * $Date: 2001/07/28 07:18:12 $
 */


#include "ReceiveBallIntention.h"
#include "Intercept.h"
#include "MoveToPosition.h"

RS_BEGIN_NAMESPACE

/** Constructs an intention. 
 */
ReceiveBallIntention::ReceiveBallIntention()
  : Intention()
{
}

/** Destroys the intention. 
 */
ReceiveBallIntention::~ReceiveBallIntention() {
}

double ReceiveBallIntention::evalWorldState(const WorldState& worldState)
{
  // Evaluate the world state.
  // Opponents close to the ball are bad.
  double oppWeight = 0.0;
  TVector ballPos = worldState.getBall().getPosition();
  const vector<PlayerState>& opponents = worldState.getOpponents();
  vector<PlayerState>::const_iterator it;
  for(it = opponents.begin(); it != opponents.end(); it++)
  {
    TVector delta = it->getPosition() - ballPos;
    oppWeight += - exp(-0.08 * delta * delta);
  }

  // Close to the opponent goal is good.
  double goalWeight = 0.1 * ballPos.getX();

  // Out of bounds is very, very bad.
  double oobWeight = 0.0;
  // FIXME: Ugly hack...
  if( (abs(ballPos.getY()) > parameterView->SP_pitch_width / 2)
      || (abs(ballPos.getX()) > parameterView->SP_pitch_length / 2))
  {
    oobWeight = -10;
  }

  // Low distance is good. (Remaining stamina is high...)
  TVector delta = ballPos - worldState.getAgent().getPosition();
  double distWeight = exp(-0.08 * delta * delta);

  return oppWeight + goalWeight + oobWeight + distWeight;
}

// Uncomment either chooseSkill or generatePlan
/* Generates a plan. 
 * Returns an empty plan.
 */
Plan ReceiveBallIntention::generatePlan() {
  static Intercept intercept(20);
  static MoveToPosition move;

//   cout << "ReceiveBallIntention.generatePlan()" << endl;
  // Check n future worldstates.
  const WorldState& firstWs = worldStateView->getWorldState();
  WorldState ws = firstWs;
  double bestValue = -100000000.0;
  TVector bestInterceptPos = ws.getBall().getPosition();
  int bestInterceptTime = 0;

  // Find the best interceptable ball position.
#define NUM_STATES 20
  for(int i = 0; i < NUM_STATES; i++)
  {
    if(firstWs.getAgent().canIntercept(ws.getBall()))
    {
      double curEval = evalWorldState(ws);
      if(curEval > bestValue)
      {
	bestValue = curEval;
	bestInterceptPos = ws.getBall().getPosition();
	bestInterceptTime = ws.getGameTime();
      }
    }
    ws.step();
  }

  // Heuristics once again.
  TVector delta = bestInterceptPos - firstWs.getAgent().getPosition();
  int deltaTime = bestInterceptTime - firstWs.getGameTime();

  // If we are close to the position, don't move.
  if(delta.getLength() < 0.8 * parameterView->SP_kickable_area)
  { 
    // Wait...
    ws = firstWs;
    ws.doTurnTowardsBall();
    ws.doTurnNeckTowardsBall();
    Plan p = Plan(ws.getStep());
    return p;
  }
  
  // Check if we can move slowly...
  if(delta.getLength() 
     < (0.5 * deltaTime * parameterView->SP_player_speed_max))
  {
    move.setPosition(bestInterceptPos);
    return move.generatePlan();
  }

  if(intercept.applicable())
  {
    return intercept.generatePlan();
  }

//   cout << "Panic, ReceiveBallIntention shouldn't be applicable!" << endl;

  return Plan(); // Your plan
}

// Uncomment either chooseSkill or generatePlan
/* Chooses a skill. It works like a mini-decision maker.
 *
 Skill* ReceiveBallIntention::chooseSkill() {
 return NULL; // Your skill
 }
*/

/** Models a player. 
 * This intention simply calls plr.doNull().
 * @param plr the player to be modelled
 */
void ReceiveBallIntention::model(WorldState& ws) {
  ws.doNull(); // Do nothing
}

/** Exaggerates a value, default is return d*1.5 + 2
 */
double ReceiveBallIntention::exaggerate(double d)
{
  return d*1.5 + 2.0;
}

/** Finds the number of players in the vector that are either
 * better (has an intercept time below our exaggerated intercept time)
 * or similar (has an intercept time below our real intercept time). 
 */
void ReceiveBallIntention::findBetter(int plrTime, 
				      const BallState& ball, 
				      const vector<PlayerState>& plrs, 
				      int& better, int& similar)
{ 
  better = 0;
  similar = 0;
  vector<PlayerState>::const_iterator it;
  for(it = plrs.begin(); it != plrs.end(); it++)
  {
    int curTime = it->getInterceptTime(ball);
    if(curTime < plrTime)
    {
      similar++;
    }

    int overTime = int(exaggerate(curTime));
    if(overTime < plrTime)
    {
      better++;
    }
  }
}

/** Checks if the utility of this intention.
 */
Intention::Utility ReceiveBallIntention::utility() {

  return Intention::uSuck;

  const WorldState& worldState = worldStateView->getWorldState();
  const BallState& ball = worldState.getBall();
  const AgentState& agent = worldState.getAgent();

  // Check which team kicked the ball last.
  //    WorldState::wsTeam lastKickTeam = worldState.getLastKickerTeam();
  // Micke: last kicker moved to statisticsView...
  
  
  // Time to check who can intercept the ball first.
  // We'll use heuristics.
  int myTime = agent.getInterceptTime(ball);

  int numTeammatesSimilar = 0;
  int numTeammatesBetter = 0;
  findBetter(myTime, 
	     ball, 
	     worldState.getTeammates(),
	     numTeammatesBetter,
	     numTeammatesSimilar);

  int numOpponentsBetter = 0;
  int numOpponentsSimilar = 0;
  findBetter(myTime, 
	     ball, 
	     worldState.getOpponents(),
	     numTeammatesBetter,
	     numTeammatesSimilar);

  // Add communicated impressions and stuff.

  // Evaluate the utility.

  // Check if there are opponents that are closer than me.
  if(numOpponentsBetter > 0)
  {
    if(numTeammatesBetter > 0)
    {
      return Intention::uVeryBad;
    }
    else
    {
      if(numTeammatesSimilar > 1)
      { 
	return Intention::uVeryBad;
      }
      else
      {
	return Intention::uBad;
      }
    }
  } 

  // No opponents are closer than me.
  // Check if there are opponents that are at similar distances as me.
  if(numOpponentsSimilar > 0)
  {
    if(numTeammatesBetter > 0)
    {
      return Intention::uBad;
    }
    else
    {
      if(numTeammatesSimilar > 1)
      {
	return Intention::uBad;
      }
      else
      {
	return Intention::uNormal;
      }
    }
  }

  // No opponents are closer than me.
  // No opponents are at similar distances as me.

  if(numTeammatesBetter > 0)
  {
    if(numTeammatesBetter > 1)
    {
      return Intention::uBad;
    }
    else
    {
      if(numTeammatesSimilar > 0)
      {
	return Intention::uBad;
      }
      else
      {
	return Intention::uNormal;
      }
    }
  }

  // No opponents are closer than me.
  // No opponents are at similar distances as me.
  // No teammates are closer than me.

  if(numTeammatesSimilar > 0)
  {
    if(numTeammatesSimilar > 1)
    {
      return Intention::uNormal;
    }
    else
    {
      return Intention::uGood;
    }
  }

  // No opponents are closer than me.
  // No opponents are at similar distances as me.
  // No teammates are closer than me.
  // No teammates are at similar distances as me.

  return Intention::uVeryGood;
}

Intention::Utility ReceiveBallIntention::modelUtility(const WorldState&)
{
  return Intention::uSuck;
}

/**
 */
float ReceiveBallIntention::ballIsHeadedTo(const PlayerState& plr) {
  const BallState& ball(worldStateView->getWorldState().getBall());

  if(plr.isBallKickable(ball))
    return 0; // The player already player has the ball

  TLine ballLine(ball.getPosition(),
		 ball.getPosition() + ball.getSpeed());
  TVector crossVec(ballLine.getVector());
  crossVec.rotate(90);
  crossVec.setLength(parameterView->SP_kickable_area);
  TLine playerLine(plr.getPosition(),
		   plr.getPosition() + crossVec);

  double ballScale,playerScale;
  crossVec = ballLine.getIntersection(playerLine, ballScale, playerScale);

  // xxxScale means: < 0, behind line, > 1 in front of line, otherwise on line.
  // In the case of playerScale, positive values mean left the player, and
  // negative mean right. > 1 is outside the kickable area.

  // *** Get ball value ***
  static const double maxDiff((1-pow(parameterView->SP_ball_decay,30))/
			      (1-parameterView->SP_ball_decay));
  static const double minDiff(-((1-pow(parameterView->SP_ball_decay,15))/
				(1-parameterView->SP_ball_decay)));
  static const double zeroValue(0.8);
  static const double slowPlayerFactor(1.6); 
  
  double diff = ballScale - fabs(playerScale) * slowPlayerFactor;
  // Linear interpolation between:
  // (minDiff,0), (0,zeroValue), (maxDiff,1)
  double getBallValue(0);
  if(diff < minDiff)
    getBallValue = 0;
  else if(diff < 0)
    getBallValue = (diff - minDiff) / (0 - minDiff) * zeroValue;
  else if(diff < maxDiff)
    getBallValue = (diff - 0) / (maxDiff - 0) * (1 - zeroValue) + zeroValue;
  else 
    getBallValue = 1.0;
  
  // *** Distance value ***
  // The distance value is like a mask, that masks out (is multiplied with) 
  // the get ball value
  static const double maxBallScale((1-pow(parameterView->SP_ball_decay,30))/
				   (1-parameterView->SP_ball_decay));
  static const double maxBehindBallDist(3.0);
  const double ballDist((ball.getPosition() - plr.getPosition()).getLength());
  double distanceValue(0);
  if(ballScale < 1) {
    // Ball is behind the player (or will be) and is moving further away
    distanceValue = minMax(0.0,
			   1.0 - (ballDist - parameterView->SP_kickable_area)/
			   (maxBehindBallDist - 
			    parameterView->SP_kickable_area),
			   1.0);
  } else {
    // Ball is headed to the player
    
    distanceValue = minMax(0.0,
			   (maxBallScale * 1.10 - ballScale)/
			   (maxBallScale * 0.20),
			   1.0);
    // The statement above is the same as below, with some fuzzyness
    //      distanceValue = (ballScale > maxBallScale ? 0.0 : 1.0 );
  }
  return getBallValue * distanceValue;
}

/** Returns the persistence of this intention.
 * This intention is very persistent.
 * @return a float between zero (not persistent) and one (very persistent)
 */
float ReceiveBallIntention::getPersistence() const {
  return 1.0; // Very persistent
}

/** Returns the ID that identifies this (type of) intention.
 */
IntentionId ReceiveBallIntention::getID() const {
  return RECEIVE_BALL_INTENTION_ID; // Add a line in "intention_constants.h" and return that.
}

RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "ReceiveBallIntention.icc"
#endif
