/* -*- Mode: C++ -*-
 * $Id: SkillGoToPosition.h,v 1.1.1.1 2001/04/08 11:33:58 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: SkillGoToPosition.h
 * Unitname: Framework/skill
 * $Revision: 1.1.1.1 $
 * Created by: Fredrik Heintz 2000-09-07
 * Last modified by $Author: sigge $ $Date: 2001/04/08 11:33:58 $
 *
 *
 * HISTORY
 *
 * $Log: SkillGoToPosition.h,v $
 * Revision 1.1.1.1  2001/04/08 11:33:58  sigge
 * Imported sources
 *
 * Revision 1.2  2000/10/07 18:02:34  perpe427
 * Restructured inheritage for MoveToPositionFacing*-skills
 *
 * Revision 1.1  2000/10/07 14:28:26  olalu526
 * Added some skill files and fixed things.
 *
 *
 */

/**
 * @file SkillGoToPosition.h
 * Include file for the skill class GotoBall.
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:58 $
 */


#ifndef SKILLGOTOPOSITION_H
#define SKILLGOTOPOSITION_H


#include "Skill.h"


RS_BEGIN_NAMESPACE


/**
 * @class SkillGoToPosition SkillGoToPosition.h
 * The first sentence is a brief description of the skill class SkillGoToPosition.
 * The rest is a longer description of the skill class SkillGoToPosition.
 * Use it to describe the skill!
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Framework/skill
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:58 $
 */
class SkillGoToPosition : public Skill {
public:
  /** Default constructor. Here you can initialize the skill. */
  SkillGoToPosition();

  /** Destructor should clean up after the skill. */
  virtual ~SkillGoToPosition();

  
  /** This method is used to generate a plan for how the goal of
    the skill should be obtained by the agent. You can assume
    that the preconditions of the skill are true (this can be tested
    by calling the applicable() method, after you have implemented it). */
  virtual Plan generatePlan() = 0;

  /** This method should return a FuzzyBool telling the caller
    if the skill wants to be called again. */
  virtual FuzzyBool persistent() = 0;

  /** This method should implement the precondition specified in
   the description of the skill. */
  virtual FuzzyBool applicable() = 0;

  /** Sets the minimum stamina when having reached the position. */
  virtual void setMinStaminaWhenFinished(Float);

  /** Gets the minimum stamina when having reached the position. */
  virtual Float getMinStaminaWhenFinished() const;

  /** Sets the stamina when having reached the position. */
  virtual void setMaxStaminaUsedWhenFinished(Float);

  /** Gets the stamina when having reached the position. */
  virtual Float getMaxStaminaUsedWhenFinished() const;

  /** Gets the number of cycles it takes to reach the position. */
  virtual UInt cyclesToReach();

  /** Gets the number of cycles it takes to reach the position. */
  virtual Float timeToReach();

  /** Gets the number of cycles it takes to reach the position. */
  virtual FuzzyBool reachable(UInt);

  /** Gets the number of cycles it takes to reach the position. */
  virtual FuzzyBool reachable(Point, UInt);

  virtual void setFaceAngle(const AngleDeg&);
  virtual AngleDeg getFaceAngle() const;

private:
  AngleDeg angle;
  Float minStaminaWhenFinished;

  /** Hide copy-constructor. */
  SkillGoToPosition(const SkillGoToPosition&);
  /** Hide assignment operator. */
  SkillGoToPosition& operator= (const SkillGoToPosition&);
};


RS_END_NAMESPACE


// Remove from here 
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "SkillGoToPosition.icc"
#endif
// To here if you do not have any inline definitions!

#endif // !SKILLGOTOPOSITION_H
