/* -*- Mode: C++ -*-
 * $Id: FCFooDecision.h,v 1.10 2001/04/23 10:05:12 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FCFooDecision.h
 * Unitname: Framework/example/FCFoo
 * $Revision: 1.10 $
 * Created by: Fredrik Heintz 1999-xx-xx
 * Last modified by $Author: mikbr212 $ $Date: 2001/04/23 10:05:12 $
 *
 *
 * HISTORY
 *
 * $Log: FCFooDecision.h,v $
 * Revision 1.10  2001/04/23 10:05:12  mikbr212
 * removed old views
 *
 * Revision 1.9  2001/04/22 16:56:21  mikbr212
 * removed old views such as agentView etc..
 *
 * Revision 1.8  2001/04/21 14:44:37  sigge
 * Removed a whole bunch of skills from the Makefiles. Now the agent should be free of fuzzy views.
 *
 * Revision 1.7  2001/04/20 14:45:05  sigge
 * Modified GameView to have a vector of PlayModes. Is not currently used, yet.
 *
 * Revision 1.6  2001/04/20 12:28:39  sigge
 * Restructured include files. Now uses the decision tree.
 *
 * Revision 1.5  2001/04/18 14:43:37  mikbr212
 * uses passtree
 *
 * Revision 1.4  2001/04/13 16:28:09  sigge
 * Added communication
 *
 * Revision 1.3  2001/04/13 11:17:44  mikbr212
 * changed display
 *
 * Revision 1.2  2001/04/11 16:24:56  andbo928
 * It now uses Display.
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.2  2000/09/05 21:12:55  frehe
 * Made FCFoo compile with RoboSoc 2.5.0
 *
 * Revision 1.1.1.1  2000/08/31 13:26:05  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 */

/**
 * @file FCFooDecision.h
 * Include file for the class FCFooDecision.
 *
 * @par Unit:
 *      Framework/example/FCFoo
 * $Revision: 1.10 $
 * $Author: mikbr212 $
 * $Date: 2001/04/23 10:05:12 $
 */


#ifndef FCFOODECISION_H
#define FCFOODECISION_H


#define RS_USE_DISPLAY
//#define RS_USE_COACH_DEBUGGING


#include "basic_system_compiler_directives.h"
#include "Decision.h"
#include "ViewManager.h"
#include "Skill.h"
#include "PlanContainer.h"

// Views
#include "CommunicationView.h"
#include "GameView.h"
#include "LookView.h"
#include "MarkerView.h"
#include "ParameterView.h"
#include "PassTreeView.h"
#include "WorldStateView.h"

#ifdef RS_USE_DISPLAY
#  include "Display.h"
#endif

#ifdef RS_USE_COACH_DEBUGGING
//  #  include "CoachView.h"
#endif


RS_BEGIN_NAMESPACE


/**
 * @class FCFooDecision FCFooDecision.h
 * Example class for what a decision maker might look like.
 *
 * <!-- @todo-->
 *
 * @par Unit:
 *      Library
 * $Revision: 1.10 $
 * $Author: mikbr212 $
 * $Date: 2001/04/23 10:05:12 $
 */
class FCFooDecision : public Decision {
public:
  FCFooDecision(ActuatorInterface* const actuator_interface = NULL);
  virtual ~FCFooDecision();

  virtual ServerCommand* initAgent();
  virtual bool start();
  //virtual bool init(ActuatorInterface* const actuator_interface);
  virtual bool decide();

  // Event handlers
  virtual void onActuatorSensorData();
  //virtual void onAuralSensorData();
  virtual void onInit();
  //virtual void onPhysicalSensorData();
  virtual void onVisualSensorData();

  //virtual void onBeforeSensorData();
  virtual void onAfterSensorData();
  //virtual void onBeforeTick();
  //virtual void onAfterTick();

  virtual void onNewCycle(const bool forced);
  virtual bool onEstimatedNewCycle(UInt16 estimated_time, UInt16 ticks);
  virtual void onCommandWarning();

protected:
  Skill* chooseSkill();
  Vector getGoalVector();

  ViewManager* const viewManager;
  ParameterView* parameterView;
  GameView* gameView;
  MarkerView* markerView;
  CommunicationView* communicationView;
  WorldStateView* worldStateView;
  LookView* lookView;
  PassTreeView* passTreeView;
  bool serverAlive;

  bool busy;

#ifdef RS_USE_DISPLAY
  Display* theDisplay;
#endif

#ifdef RS_USE_COACH_DEBUGGING
//    CoachView* coachView;
#endif
};


RS_END_NAMESPACE


#endif // !FCFOODECISION_H
