/* -*- Mode: C++ -*-
 * $Id: CommunicationView.cc,v 1.7 2001/04/24 09:54:47 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CommunicationView.cc
 * Unitname: Framework/view
 * $Revision: 1.7 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: sigge $ $Date: 2001/04/24 09:54:47 $
 *
 *
 * HISTORY
 *
 * $Log: CommunicationView.cc,v $
 * Revision 1.7  2001/04/24 09:54:47  sigge
 * Moved communication policy to communication view. Communication view no longer erases messages not picked up from the incoming queue in order to be useful to skills.
 *
 * Revision 1.6  2001/04/21 14:44:11  sigge
 * Removed a whole bunch of skills from the Makefiles. Now the agent should be free of fuzzy views.
 *
 * Revision 1.5  2001/04/18 13:50:20  sigge
 * Fixed another problem with communication (for sizes of over 256).
 *
 * Revision 1.4  2001/04/17 14:11:44  sigge
 * Added team recognition to avoid crashing due to other teams messages.
 *
 * Revision 1.3  2001/04/15 23:18:52  andbo928
 * Disabled due to bugs.
 *
 * Revision 1.2  2001/04/13 16:27:10  sigge
 * Reimplemented base64Encoder and did the myUpdateAfterHear() properly.
 *
 * Revision 1.1  2001/04/12 13:23:20  sigge
 * Added CommunicationView
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.8  2000/10/28 20:22:02  mikbr212
 * changed initBall: now ballSpeed is retrieved from commandView if not known in ballView
 *
 * Revision 1.7  2000/10/26 16:00:45  mikbr212
 * Fixed bug in kickPowerRate: kickable_area to kickable_margin
 *
 * Revision 1.6  2000/10/09 18:46:15  mikbr212
 * Fixed serious wrong-calculations in getKickVector
 *
 * Revision 1.5  2000/10/09 10:17:44  mikbr212
 * added comments
 *
 * Revision 1.4  2000/10/08 20:05:01  mikbr212
 * Removed spam messages
 *
 * Revision 1.3  2000/10/08 12:57:28  mikbr212
 * removed comment
 *
 * Revision 1.2  2000/10/07 19:42:01  olalu526
 * Uncommented part using line.
 *
 * Revision 1.1  2000/10/07 14:29:25  mikbr212
 * created
 *
 */

/**
 * @file CommunicationView.cc
 * Implementation file for the view class CommunicationView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.7 $
 * $Author: sigge $
 * $Date: 2001/04/24 09:54:47 $
 */


#include "CommunicationView.h"
#include "SensorInterface.h"
#include "FrameworkSensorInterface.h"
#include "WorldState.h"

RS_BEGIN_NAMESPACE

/**
 * Constructor. Use getView instead.
 * @see getView
 */
CommunicationView::CommunicationView(const ViewId id)
  : View(id)
{
}

CommunicationView::~CommunicationView()
{
}

void CommunicationView::myNewCycle()
{
}

void CommunicationView::myUpdateAfterCoachMessage(void)
{
}

void CommunicationView::myUpdateAfterSense(void)
{
}

void CommunicationView::myUpdateAfterSee(void)
{
}

void CommunicationView::myUpdateAfterInit(void)
{
}

void CommunicationView::myUpdateAfterHear(void)
{
  string message = sensorInterface->getPlayerMessage();
  if(message.size() > 0)
  {
    //     cout << "Message: '" << message << "'" << endl;
    // message is " <direction> <Message>)\n"
    // strip first space
    string realMessage = message.substr(1);
    // strip direction
    realMessage = realMessage.substr(realMessage.find(" "));
    // strip second space
    realMessage = realMessage.substr(1);
    // remove trailing ')\n'
    realMessage = realMessage.substr(0, realMessage.size()-2);
    //     cout << "realMessage: '" << realMessage << "'" << endl;
    // Check for "validity"
    if(realMessage.substr(0, 8).compare("FuzzyFoo") == 0)
    {
      // remove "FuzzyFoo"
      realMessage = realMessage.substr(8);

      string s256 = base64Decode(realMessage);
      //     cout << "s256: '" << s256 << "'" << endl;
      int numMessages = 0;
      while(s256.size() > 0)
      {
	try
	{
	  Message msg(s256);
	  incomingMessages.push_back(msg);
	  numMessages++;
	}
	catch (std::exception& ex)
	{
	  cout << "Message decoding failed, numMessages=" << numMessages << endl;
// 	  cout << "message: '" << message << "'" << endl;
	}
      }
    }
  }
}

void CommunicationView::myUpdateAfterCommand(void)
{
}

string CommunicationView::getOutgoing()
{
  string concat = "";
  if(outgoingMessages.size() > 1)
    cout << "CommunicationView::getOutgoing(), size=" << outgoingMessages.size() << endl;
  for(unsigned int i=0; i<outgoingMessages.size(); i++)
    concat += outgoingMessages[i].serialize();
  outgoingMessages.clear();
  return "FuzzyFoo" + base64Encode(concat);
}

void CommunicationView::addMessage(const Message message)
{
  outgoingMessages.push_back(message);
}

vector<Message> CommunicationView::getMessages(int type)
{
  vector<Message> res;
  for(vector<Message>::iterator msg=incomingMessages.begin();
      msg != incomingMessages.end(); /* empty */)
    if(msg->getType() == type)
    {
      res.push_back(*msg);
      msg = incomingMessages.erase(msg);
    }
    else
      msg++;
  return res;
}


string CommunicationView::base64Encode(const string source)
{
  string res;
  int bitCount = 0;

  unsigned int i = 0;
  while (i < source.size())
  {
    unsigned char b64;
    if (bitCount <= 2)
    {
      // Just grab it
      b64 = (((unsigned char)source[i]) >> (2-bitCount));
    }
    else
    {
      // divide the grab into two parts
      b64 = 0;
      // high part:
      b64 |= (((unsigned char)source[i]) << (bitCount - 2));
      // low part, might be outside of string
      unsigned char b256;
      if( i == source.size() - 1 )
	b256 = 0x00;
      else
	b256 = (unsigned char)source[i+1];
      b64 |= (b256 >> (10 - bitCount));
    }
    if(bitCount)
      i++;
    bitCount = (bitCount + 6) & 0x7;
    b64 &= 0x3f;
    res += legalChars[b64];
  }

  return res;
}

string CommunicationView::base64Decode(string source)
{
  string res;

  for(int i = 0; i < 64; i++)
    {
      lookUp[(unsigned char)legalChars[i]] = i;
    }

  int bitCounter = 0;
  int wrd = 0;
  unsigned int i = 0;
  int sourceIndex = 0;
  while(i < source.size())
    {
      while(bitCounter < 8)
	{
	  wrd <<= 6;
	  wrd |= lookUp[source[sourceIndex]];
	  bitCounter += 6;
	  sourceIndex++;
	  i++;
	}
      int b256 = (wrd >> (bitCounter-8)) & 0xff;
      wrd &= (1 << (bitCounter-8)) - 1;
      bitCounter -= 8;

      res += b256;
    }
  return res;
}

bool CommunicationView::isThisMySpeakCycle(const WorldState& worldState)
{
  // Decide if we should talk
  // Simple round robin, send every 22nd cycle
  int uniformNumber = worldState.getAgent().getUniformNumber();

  if(uniformNumber > 0)
  {
    int gameTime = worldState.getGameTime();
    if( (gameTime % 22) == 2*(uniformNumber-1) )
      return true;
  }
  return false;
}

RS_END_NAMESPACE
