/* -*- Mode: C++ -*-
 * $Id: CommunicationView.h,v 1.4 2001/04/24 09:54:47 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CommunicationView.h
 * Unitname: Framework/view
 * $Revision: 1.4 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: sigge $ $Date: 2001/04/24 09:54:47 $
 *
 *
 * HISTORY
 *
 * $Log: CommunicationView.h,v $
 * Revision 1.4  2001/04/24 09:54:47  sigge
 * Moved communication policy to communication view. Communication view no longer erases messages not picked up from the incoming queue in order to be useful to skills.
 *
 * Revision 1.3  2001/04/21 14:44:11  sigge
 * Removed a whole bunch of skills from the Makefiles. Now the agent should be free of fuzzy views.
 *
 * Revision 1.2  2001/04/15 00:06:50  andbo928
 * Changed legalchats since parenthesis wasnt legal, stupid manual.
 *
 * Revision 1.1  2001/04/12 13:23:20  sigge
 * Added CommunicationView
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.2  2000/10/28 20:22:03  mikbr212
 * changed initBall: now ballSpeed is retrieved from commandView if not known in ballView
 *
 * Revision 1.1  2000/10/07 14:29:26  mikbr212
 * created
 *
 * Revision 1.2  2000/09/17 14:17:32  frehe
 * Made minor updates to the CommunicationView.
 *
 * Revision 1.1  2000/09/13 00:06:21  frehe
 * Added a view template called CommunicationView.
 *
 */

/**
 * @file CommunicationView.h
 * Include file for the view class CommunicationView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: sigge $
 * $Date: 2001/04/24 09:54:47 $
 */


#ifndef COMMUNICATIONVIEW_H
#define COMMUNICATIONVIEW_H


#include "View.h"
#include "Message.h"

#include "view_constants.h"
#include "ViewManager.h"

RS_BEGIN_NAMESPACE

class WorldState;

// C++ er domt, so we use a nice global constant instead
const string legalChars("abcdefghijklmnopqrstuvwxyz"
			"ABCDEFGHIJKLMNOPQRSTUVWXYZ"
			"1234567890+-*/?<>_");
/**
 * @class CommunicationView CommunicationView.h
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.4 $
 * $Author: sigge $
 * $Date: 2001/04/24 09:54:47 $
 */
// If you do not want any history then inherit from View instead.
//class CommunicationView : public View {
class CommunicationView : public View {
public:
  // If you do not want history then remove the second argument.
  // Don't forget to update the description and CommunicationView.cc as well.
  // CommunicationView(const ViewId id);
  /** Construct a view with the ViewId @a id and the history length
    @a hist_len. */
  CommunicationView(const ViewId id);

  /** Destroys the view. */
  ~CommunicationView();

  /** Returns a string of what is to be communicated */
  string getOutgoing();

  void addMessage(const Message message);
  /** Searches for all messages of type @a type and returns a
      vector of the search resulta */
  vector<Message> getMessages(int type);

  bool isThisMySpeakCycle(const WorldState& worldState);

protected:
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  vector<Message> incomingMessages;
  vector<Message> outgoingMessages;

private:
  // Hide copy-constructor and assignment.
  CommunicationView(const CommunicationView&);
  CommunicationView& operator= (const CommunicationView&);

  char lookUp[256];
  string base64Encode(string source);
  string base64Decode(string source);

};


RS_END_NAMESPACE


#endif // !COMMUNICATIONVIEW_H
