/* -*- Mode: C++ -*-
 * $Id: EventView.h,v 1.3 2001/04/23 10:22:22 andbo928 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: EventView.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Mikael Brnnstrm, Andreas Sigfridsson, Andreas Bckert
 * Last modified by $Author: andbo928 $ $Date: 2001/04/23 10:22:22 $
 *
 *
 * HISTORY
 *
 * $Log: EventView.h,v $
 * Revision 1.3  2001/04/23 10:22:22  andbo928
 * Cleaned up.
 *
 * Revision 1.2  2001/04/22 14:27:02  andbo928
 * Fixed alot of stuff.
 *
 * Revision 1.1  2001/04/11 18:06:57  andbo928
 * Added eventview.
 *
 * Revision 1.1  2001/04/11 09:10:25  mikbr212
 * Created view
 *
 *
 */

/**
 * @file EventView.h
 * Implementation file for the class EventView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: andbo928 $
 * $Date: 2001/04/23 10:22:22 $
 */



#ifndef EVENTVIEW_H
#define EVENTVIEW_H

#include "ViewWithHistory.h"
#include "ParameterView.h"
//#include "CommandView.h"
#include "WorldStateView.h"
#include "view_constants.h"
#include "FuzzyBool.h"
#include "Predicate.h"

RS_BEGIN_NAMESPACE

/**
 * @class EventView EventView.h
 * This view is used to keep track of the ball.
 *
 * <!-- @invariant The class invariant is... -->
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: andbo928 $
 * $Date: 2001/04/23 10:22:22 $
 */

//  void myCallback(const Predicate& pred,
//  		FuzzyBool predRes,
//  		void* ptr)
	    

// Do not keep the pointervalue after the return!
typedef void (*callbackFn)(Predicate*,
			   FuzzyBool predRes,
			   void* ptr);

class EventView : public View {
public:
  EventView(const ViewId id);
  /** Destroys the ball view. */
  virtual ~EventView();

  void addEvent(Predicate* _predicate,
		unsigned int _startCycle,
		unsigned int _endCycle,
		void* _appPtr,
		callbackFn _callback);

protected:

  WorldStateView* worldStateView;
  
  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  vector<Predicate*> predicate;
  vector<callbackFn> callback;
  vector<unsigned int> startCycle;
  vector<unsigned int> endCycle;
  vector<void*> appPtr;

private:
  // Hide copy-constructor and assignment.
  EventView(const EventView&);
  EventView& operator= (const EventView&);  
};


RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "EventView.icc"
#endif


#endif // !EVENTVIEW_H

