/* -*- Mode: C++ -*-
 * $Id: LookView.h,v 1.6 2001/07/26 15:44:59 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: LookView.h
 * Unitname: Framework/view
 * $Revision: 1.6 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/26 15:44:59 $
 *
 *
 * HISTORY
 *
 * $Log: LookView.h,v $
 * Revision 1.6  2001/07/26 15:44:59  mikbr212
 * Computes values on a radius reduced world state now
 *
 * Revision 1.5  2001/07/06 10:00:38  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.4  2001/04/16 13:31:27  mikbr212
 * Debugged view
 *
 * Revision 1.3  2001/04/13 14:55:39  mikbr212
 * added stuff
 *
 * Revision 1.2  2001/04/13 10:23:46  mikbr212
 * Now using worldStateView
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.2  2000/10/28 20:22:03  mikbr212
 * changed initBall: now ballSpeed is retrieved from commandView if not known in ballView
 *
 * Revision 1.1  2000/10/07 14:29:26  mikbr212
 * created
 *
 * Revision 1.2  2000/09/17 14:17:32  frehe
 * Made minor updates to the LookView.
 *
 * Revision 1.1  2000/09/13 00:06:21  frehe
 * Added a view template called LookView.
 *
 */

/**
 * @file LookView.h
 * Include file for the view class LookView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/26 15:44:59 $
 */


#ifndef LOOKVIEW_H
#define LOOKVIEW_H


#include "View.h"
#include "WorldStateView.h"
#include "ParameterView.h"

#include "view_constants.h"
#include "ViewManager.h"

RS_BEGIN_NAMESPACE

/**
 * @class LookView LookView.h
 * This view keeps track of which directions the agent has
 * looked in and where the confidence needs to be improved.
 * Simply call getAngle to get the direction. 
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.6 $
 * $Author: mikbr212 $
 * $Date: 2001/07/26 15:44:59 $
 */
class LookView : public View {
public:
  /** Construct a view with the ViewId @a id. */
  LookView(const ViewId id, int _slices = 8);

  /** Destroys the view. */
  ~LookView();

  AngleDeg getAngle(AngleDeg min = AngleDeg(-179.9999),
		    AngleDeg max = AngleDeg(180));

  AngleDeg getAngle(Float& bestValue,
		    AngleDeg min = AngleDeg(-179.9999),
		    AngleDeg max = AngleDeg(180));

  void setWeights(Float ws = 0.3, Float wp = 0.3, Float wb = 0.4);
  Float getMaxValue();
protected:
  Float getSlice(AngleDeg a);
  int getIndex(AngleDeg a);
  Float indexToAngle(int i);

  void computeLookValues();

  virtual void myNewCycle();
  virtual void myUpdateAfterCoachMessage();
  virtual void myUpdateAfterInit();
  virtual void myUpdateAfterSee();
  virtual void myUpdateAfterHear();
  virtual void myUpdateAfterSense();
  virtual void myUpdateAfterCommand();

  int slices;
  std::vector<Float> see;
  std::vector<Float> lookValues;
  std::vector<Float> integralLookValues;
  
  Float wSee; // weight see
  Float wPlr; // weight player
  Float wBall; // weight ball

  WorldStateView* worldStateView;
  ParameterView* parameterView;

private:
  // Hide copy-constructor and assignment.
  LookView(const LookView&);
  LookView& operator= (const LookView&);  
};


inline
Float LookView::indexToAngle(int i) {
  return ((Float)i + 0.5 ) / (Float) slices * 360;
}

inline
AngleDeg LookView::getAngle(AngleDeg min, AngleDeg max) {
  Float bestVal;
//    cout << "LV: inline getAngle" << endl;
  return getAngle(bestVal, min, max);
}

inline
int LookView::getIndex(AngleDeg a) {
  return (int) floor(getSlice(a));
}


RS_END_NAMESPACE


#endif // !LOOKVIEW_H
