/* -*- Mode: C++ -*-
 * $Id: Message.cc,v 1.3 2001/04/18 13:50:20 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: CommunicationView.cc
 * Unitname: Framework/view
 * $Revision: 1.3 $
 * Created by: Fredrik Heintz 2000-09-12
 * Last modified by $Author: sigge $ $Date: 2001/04/18 13:50:20 $
 *
 *
 * HISTORY
 *
 * $Log: Message.cc,v $
 * Revision 1.3  2001/04/18 13:50:20  sigge
 * Fixed another problem with communication (for sizes of over 256).
 *
 * Revision 1.2  2001/04/17 16:33:46  sigge
 * Fix sign problem and catch those darn exceptions that haunt me.
 *
 * Revision 1.1  2001/04/12 13:38:18  sigge
 * Added CommunicationView
 *
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.8  2000/10/28 20:22:02  mikbr212
 * changed initBall: now ballSpeed is retrieved from commandView if not known in ballView
 *
 * Revision 1.7  2000/10/26 16:00:45  mikbr212
 * Fixed bug in kickPowerRate: kickable_area to kickable_margin
 *
 * Revision 1.6  2000/10/09 18:46:15  mikbr212
 * Fixed serious wrong-calculations in getKickVector
 *
 * Revision 1.5  2000/10/09 10:17:44  mikbr212
 * added comments
 *
 * Revision 1.4  2000/10/08 20:05:01  mikbr212
 * Removed spam messages
 *
 * Revision 1.3  2000/10/08 12:57:28  mikbr212
 * removed comment
 *
 * Revision 1.2  2000/10/07 19:42:01  olalu526
 * Uncommented part using line.
 *
 * Revision 1.1  2000/10/07 14:29:25  mikbr212
 * created
 *
 */

/**
 * @file CommunicationView.cc
 * Implementation file for the view class CommunicationView.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.3 $
 * $Author: sigge $
 * $Date: 2001/04/18 13:50:20 $
 */


#include "CommunicationView.h"
#include "Message.h"
#include <stdio.h>

RS_BEGIN_NAMESPACE

Message::Message(const int _type, const string _contents)
{
  type = _type;
  contents = _contents;
}

/** Decode the message from a bytestream. The message is encoded as:
    type (7 bits (most significant in the first byte)),
    size (9 bits), contents (size bytes).
    Will destructively change @_rawContents.
*/
Message::Message(string& _rawContents)
{
  type = _rawContents[0] >> 1;
  int size = ((unsigned char)_rawContents[1]) +
    ((((unsigned char)_rawContents[0]) & 0x1) << 8);
  try
  {
    contents = _rawContents.substr(2, size);
    _rawContents = _rawContents.substr(2+size);
  }
  catch (std::exception& ex)
  {
//     cout << "Failed to decode message!" << endl;
//     printf("Bytes: %02x %02x, size %d\n", (unsigned int)_rawContents[0], (unsigned int)_rawContents[1], _rawContents.size());
//     cout << "decoded type: " << type << ", decoded size: " << size << endl;
    throw ex;
  }
}

Message::~Message()
{
}

int Message::getType() const
{
  return type;
}

string Message::getContents() const
{
  return contents;
}

/** Encode the message to a bytestream. Encodes the message as:
    type (7 bits (most significant in the first byte)),
    size (9 bits), contents (size bytes) */
string Message::serialize() const
{
  // Ugly shit to avoid messing with streams
  unsigned char fulhack[3];
  fulhack[2] = '\0';
//   if(contents.size() > 512)
//     throw foo();
  fulhack[0] = (((unsigned char)type) << 1) | (((contents.size() & 0x100) >> 8));
  fulhack[1] = contents.size() & 0xff;
  return string((char*)fulhack) + contents;
}

RS_END_NAMESPACE
