/* -*- Mode: C++ -*-
 * $Id: PassTree.cc,v 1.41 2001/07/31 17:07:01 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassTree.cc
 * Unitname: Framework/view
 * $Revision: 1.41 $
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/31 17:07:01 $
 *
 *
 * HISTORY
 *
 * $Log: PassTree.cc,v $
 * Revision 1.41  2001/07/31 17:07:01  mikbr212
 * new improved
 *
 * Revision 1.40  2001/07/31 14:44:37  mikbr212
 * Fixed some stuff in opponentFn
 *
 * Revision 1.39  2001/07/31 14:28:46  mikbr212
 * Added angle thingy light in opponentFn
 *
 * Revision 1.38  2001/07/30 17:00:01  mikbr212
 * Debugged and fixed stupid static bug
 *
 * Revision 1.37  2001/07/30 15:25:10  mikbr212
 * Added some doxygen comments
 *
 * Revision 1.36  2001/07/30 15:08:52  mikbr212
 * Removed memory leak
 *
 * Revision 1.35  2001/07/30 14:58:52  mikbr212
 * Added setStatic option.
 *
 * Revision 1.34  2001/07/29 13:31:13  mikbr212
 * Changed a parameter in opponentFn
 *
 * Revision 1.33  2001/07/29 12:30:06  mikbr212
 * changed constant in fieldFn
 *
 * Revision 1.32  2001/07/27 14:36:04  mikbr212
 * Added score values in analysis
 *
 * Revision 1.31  2001/07/27 13:09:33  sigge
 * Fixed indexing bug.
 *
 * Revision 1.30  2001/07/26 12:33:01  mikbr212
 * fixed bug with analysisCache
 *
 * Revision 1.29  2001/07/26 12:20:46  mikbr212
 * Changed passtree completly
 *
 * Revision 1.28  2001/07/11 16:35:50  sigge
 * Modified Monitor protocol for size and decoding speed.
 *
 * Revision 1.27  2001/07/10 17:45:17  sigge
 * Optimized opponentFn. It is now less readable.
 *
 * Revision 1.26  2001/07/10 17:29:16  andbo928
 * Cosmetic changes.
 *
 * Revision 1.25  2001/07/06 10:00:38  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.24  2001/06/19 14:01:00  mikbr212
 * Fixed circular includes stuff. Somebody should check that I did it right.
 *
 * Revision 1.23  2001/04/24 22:36:05  mikbr212
 * fixed bug
 *
 * Revision 1.22  2001/04/24 22:14:46  mikbr212
 * updated getPosition
 *
 * Revision 1.21  2001/04/24 21:27:41  mikbr212
 * ??
 *
 * Revision 1.20  2001/04/24 19:13:43  mikbr212
 * No idea what i have done
 *
 * Revision 1.19  2001/04/24 17:10:00  mikbr212
 * updated a lot
 *
 * Revision 1.18  2001/04/24 14:48:15  mikbr212
 * dont avoid goalie
 *
 * Revision 1.17  2001/04/24 13:51:27  mikbr212
 * added getPassValueToPassTarget
 *
 * Revision 1.16  2001/04/24 12:36:59  mikbr212
 * fixed stuff
 *
 * Revision 1.15  2001/04/24 09:55:19  sigge
 * Added teamFn generation.
 *
 * Revision 1.14  2001/04/23 22:49:25  mikbr212
 * added some methods: getGoodGuys etc
 *
 * Revision 1.13  2001/04/23 15:51:29  mikbr212
 * added getTeamValue()
 *
 * Revision 1.12  2001/04/23 13:10:44  mikbr212
 * added scoreValues
 *
 * Revision 1.11  2001/04/23 10:44:48  mikbr212
 * fixed getPosition
 *
 * Revision 1.10  2001/04/22 14:11:26  mikbr212
 * fixed offside for rightSide-players also
 *
 * Revision 1.9  2001/04/22 12:15:54  mikbr212
 * added offside-stuff
 *
 * Revision 1.8  2001/04/21 16:26:52  mikbr212
 * fixed stuff
 *
 * Revision 1.7  2001/04/20 18:56:03  mikbr212
 * ???
 *
 * Revision 1.6  2001/04/20 12:59:13  mikbr212
 * fixed copy and assignment
 *
 * Revision 1.5  2001/04/20 12:03:41  mikbr212
 * added display
 *
 * Revision 1.4  2001/04/20 11:40:33  sigge
 * A bunch of const fixes.
 *
 * Revision 1.3  2001/04/20 09:42:02  mikbr212
 * added const and dummy-constructor
 *
 * Revision 1.2  2001/04/20 09:30:50  mikbr212
 * added copy-constructor and assignment-operator
 *
 * Revision 1.1  2001/04/19 22:16:28  mikbr212
 * added PassTree
 *
 */

/**
 * @file PassTree.cc
 * Implementation file for the view class PassTree.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.41 $
 * $Author: mikbr212 $
 * $Date: 2001/07/31 17:07:01 $
 */

#include "PassTree.h"

#include "Display.h"
#include "PlayerState.h"
#include "BallState.h"

RS_BEGIN_NAMESPACE

/** This is just an dummy constructor to be able to create 
 * empty pass trees.
 */
// FIXME: check me
PassTree::PassTree() 
  : dirty(dClean),
    agentIndex(-1),
    teamValue(-1),
    analysisCache(NULL),
    staticTree(false),
    indices(11,-1),
    indicesBuilt(false)
{
}

/** Creates a pass tree. The last player among the teammates is chosen
 * as the agent. 
 * @param addAgent if true, then the agent is added last in the teammates
 * vector (and is also chosen as agent).
 * @see build()
 * @see promotePassWay(const PassWay&)
 */
PassTree::PassTree(const WorldState& world,
		   bool addAgent)
  : dirty(dAllPassesAndTree),
    tm(world.getTeammates()),
    op(world.getOpponents()),
    teamValue(-1),
    analysisCache(NULL),
    staticTree(false),
    indices(11,-1),
    indicesBuilt(false)
{
  if(addAgent) 
    tm.push_back(world.getAgent());

  agentIndex = int(tm.size()) - 1;
 
  initNodes(world.getBall());

  // create matrix
  std::vector<float> row;
  row.assign(tm.size(),0);
  promotedPassValues.assign(tm.size(),row); 
}

PassTree::PassTree(const PassTree& other) 
  :  dirty(other.dirty),
     agentIndex(other.agentIndex),
     tm(other.tm),
     op(other.op), 
     allNodes(other.allNodes), 
     promotedPassValues(other.promotedPassValues),
     passValues(other.passValues),
     teamValue(other.teamValue),
     analysisCache(NULL),
     staticTree(other.staticTree),
     indices(other.indices),
     indicesBuilt(other.indicesBuilt)
{
  // Pointers should point at the local tm-vector
  std::vector<PlayerNode>::iterator it;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    it->player = &tm[it->index];
  }

  if(other.analysisCache != NULL)
    analysisCache = new Analysis(*(other.analysisCache)); // copy
}

PassTree& PassTree::operator=(const PassTree& other) {
  dirty = other.dirty;
  agentIndex = other.agentIndex;
  tm = other.tm;
  op = other.op; 
  allNodes = other.allNodes; 
  promotedPassValues = other.promotedPassValues;
  passValues = other.passValues;
  teamValue = other.teamValue;
  analysisCache = NULL;
  staticTree = other.staticTree;
  
  indices = other.indices;
  indicesBuilt = other.indicesBuilt;

  // Pointers should point at the local tm-vector
  std::vector<PlayerNode>::iterator it;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    it->player = &tm[it->index];
  }

  if(other.analysisCache != NULL)
    analysisCache = new Analysis(*(other.analysisCache)); // copy

  return *this;
}


PassTree::~PassTree()
{
  // clear analysis cache
  if(analysisCache != NULL) {
    delete analysisCache;
    analysisCache = NULL;
  }
}

/** Sets if the tree should be static or dynamic (default).
 * When using a static tree, the promoted pass ways are
 * used as allowed pass ways instead. If there isn't a
 * a pass way between two players, then that pass way is
 * not allowed. The pass values are also weighted with the 
 * promoted pass values.
 */
void PassTree::setStatic(bool flag) {
  staticTree = flag;
}

/** (Re)Builds the pass tree if needed.
 */
void PassTree::build() {
  // Check passes
  if(dirty == dAllPassesAndTree) {
    buildPassValues();
    dirty = dTreeOnly;
  } else if(dirty == dAgentPassesAndTree) {
    rebuildPassValues(agentIndex);
    dirty = dTreeOnly;
  }

  // Check tree
  if(dirty == dTreeOnly) {
    buildTree();
    dirty = dClean;
  }

  // Check team value
  if(teamValue == -1) {
    teamValue = teamFn();
  }
}

/** Changes the agent and (re)builds the pass tree if needed.
 * @param _agentIndex the index of the new agent in the teammates
 * vector.
 * @see getTeammates()
 */
void PassTree::build(int _agentIndex) {
  if(agentIndex == _agentIndex) {
    build();
    return;
  }
  
  if(0 <= _agentIndex && _agentIndex < (int)tm.size()) {
    if(dirty == dAgentPassesAndTree) {
      rebuildPassValues(agentIndex);
      dirty = dTreeOnly;
    } 
    agentIndex = _agentIndex;
    build();

    // clear analysis cache
    if(analysisCache != NULL) {
      delete analysisCache;
      analysisCache = NULL;
    }
  }
}

/** Checks if the tree is completly built. This can be used
 * to check if getPassWays and getAnalysis returns a valid
 * object when using the const method.
 */
bool PassTree::isBuilt() const {
  return dirty == dClean;
}

/** Returns the team value, a value between 0 and 1 that
 * represents how good the agent and the team is positioned. 
 * The tree will be rebuilt if needed.
 */
float PassTree::getTeamValue() {
  if(teamValue == -1)
    build();
  return teamValue;
}

/** Returns the team value, a value between 0 and 1 that
 * represents how good the agent and the team is positioned. 
 * If the tree needs to be rebuilt then -1 is returned (since
 * the tree cannot be rebuilt as this is a const method).
 */
float PassTree::getTeamValue() const {
  return teamValue;
}

/** This will add a value to the given pass way if it exists.
 */
void PassTree::promotePassWay(const PassTree::PassWay& passWay, 
			      float value) {
  // First build the translation vector uNum -> index
  if(!indicesBuilt) {
    int uNum;
    unsigned int i;
    for(i = 0; i < tm.size(); i++) {
      uNum = tm[i].getUniformNumber();
      if(1 <= uNum && uNum <= 11) 
	indices[uNum-1] = int(i);
    }
    indicesBuilt = true;
  }

  if(!passWay.isValid())
    return;

  int fromIndex(indices[passWay.getFromUniformNumber()-1]);
  int toIndex(indices[passWay.getToUniformNumber()-1]);
  if(fromIndex != -1 && toIndex != -1) {
    promotedPassValues[fromIndex][toIndex] += value;
    dirty = dAllPassesAndTree;
    teamValue = -1; // invalidate

    // clear analysis cache
    if(analysisCache != NULL) {
      delete analysisCache;
      analysisCache = NULL;
    }
  }
}

/** Iterates through all passWays and simply calls promotePassWay
 * for each of them.
 */
void PassTree::promotePassWays(const std::vector<PassTree::PassWay>& passWays,
			       float value) {
  std::vector<PassTree::PassWay>::const_iterator it;
  for(it = passWays.begin(); it != passWays.end(); it++) 
    promotePassWay(*it, value);
}

/** Builds all pass values. 
 */
void PassTree::buildPassValues() {
  { // Rebuild matrix
    std::vector<float> row;
    row.assign(tm.size(),0);
    passValues.assign(tm.size(),row); 
  }

  unsigned int i,j;

  // calculate distance-value (it is symmetric)
  for(i = 0; i < tm.size(); i++) {
    for(j = 0; j < i; j++) {
      passValues[i][j] = passValues[j][i] = distFn(tm[i].getPosition(),
						   tm[j].getPosition());
      // calculate angle-to-opponents-value for non-zero distance-values
      if(passValues[i][j] > 0) {
	passValues[i][j] *= opponentFn(tm[i].getPosition(),
				       tm[j].getPosition());
	passValues[j][i] *= opponentFn(tm[j].getPosition(),
				       tm[i].getPosition());
      }
    }
  }

  if(staticTree) {
    // Just allow promoted pass ways
    for(i = 0; i < tm.size(); i++)
      for(j = 0; j < tm.size(); j++)
	passValues[i][j] *= promotedPassValues[i][j];
  } else {
    // Add promoted pass values
    for(i = 0; i < tm.size(); i++)
      for(j = 0; j < tm.size(); j++)
	passValues[i][j] += promotedPassValues[i][j];
  }
}

/** Rebuilds pass values for the given player only.
 */
void PassTree::rebuildPassValues(int playerIndex) {
  if(playerIndex >= int(tm.size()) || playerIndex < 0)
    return;

  unsigned int i,j;
  j = (unsigned int)playerIndex;

  // calculate distance-value (it is symmetric)
  for(i = 0; i < tm.size(); i++) {
    if(j != i) {
      passValues[i][j] = passValues[j][i] = distFn(tm[i].getPosition(),
						   tm[j].getPosition());

      // calculate angle-to-opponents-value for non-zero distance-values
      if(passValues[i][j] > 0) {
	passValues[i][j] *= opponentFn(tm[i].getPosition(),
				       tm[j].getPosition());
	passValues[j][i] *= opponentFn(tm[j].getPosition(),
				       tm[i].getPosition());
      }
    }
  }

  if(staticTree) {
    // Just allow promoted pass ways
    for(i = 0; i < tm.size(); i++) {
      passValues[i][j] *= promotedPassValues[i][j];
      if(i != j)
	passValues[j][i] *= promotedPassValues[j][i];
    }
  } else {
    // Add promoted pass values
    for(i = 0; i < tm.size(); i++) {
      passValues[i][j] += promotedPassValues[i][j];
      passValues[j][i] += promotedPassValues[j][i];
    }
    // Remove the value that was added twice
    passValues[j][j] -= promotedPassValues[j][j]; 
  }

}

/** Initializes all nodes and finds the root players. This will be
 * done only once, even if the agent is moved (as in evaluatePosition).
 */
void PassTree::initNodes(const BallState& ball) {
  // Create all nodes
  {
    std::vector<PlayerState>::iterator it;
    int i=0;
    for(it = tm.begin(); it != tm.end(); it++) {
      allNodes.push_back(PlayerNode(it, i)); 
      i++;
    }
  }

  // Find and init the root-node(s)
  BallState newBall(ball);
  newBall.step(2);
  TVector rootPosition(newBall.getPosition());
  float closestDistance = 99999;
  float distance = 0;
  std::vector<PlayerNode>::iterator it;
  // Calculate the closest distance
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    distance = (it->player->getPosition() - rootPosition).getLength();
    if(distance < closestDistance) 
      closestDistance = distance;
  }
  // Init all players that are within the margin
  closestDistance = min(closestDistance*1.1, closestDistance+1.0);
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    distance = (it->player->getPosition() - rootPosition).getLength();
    if(distance < closestDistance) {
      it->value = 1;
      it->parentIndex = -1; 
    }
  }
}

/** Builds the tree using Dijkstra.
 */
void PassTree::buildTree() {
  if(agentIndex == -1)
    return;

  // Init queue and clear nodes
  std::vector<PlayerNode*> q;
  std::vector<PlayerNode>::iterator it;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    if(it->parentIndex == -1) { // root player
      it->value = 1; // should not be needed
      q.push_back(it);
    } else { // non-root player
      it->parentIndex = it->index;
      it->value = 0;
    }
  }
  
  // Do the Dijkstra
  PlayerNode* current;
  make_heap(q.begin(),q.end());
  float value;
  while(!q.empty()) {
    current = dequeue(q);
    // for all children to current
    std::vector<PlayerNode>::iterator child;
    for(child = allNodes.begin(); child != allNodes.end(); child++) {
      if(child == current) 
	continue;
      value = min(current->value,
		  passValues[current->index][child->index]) * 0.99;
      if(child->value < value) { // || child->index == child->parentIndex) {
	child->value = value;
	child->parentIndex = current->index;
	enqueue(q, child);
      }
    }
  }

  // Not a tree yet... make all nodes connected to the tree
  removeOrphans();
}

/** Dequeues from the priority queue. If there are duplicates of the
 * removed value, they are also removed.
 */
PassTree::PlayerNode* PassTree::dequeue(std::vector<PlayerNode*>& q) {
  pop_heap(q.begin(),q.end());
  PlayerNode* retVal = q.back();
  q.pop_back();
  // remove duplicates
  while(!q.empty() && q.front() == retVal) {
    pop_heap(q.begin(),q.end());
    q.pop_back();
  }
  return retVal;
}

/** Enqueues a player node on the priority queue. Duplicates are
 * handled in dequeue(..).
 */
void PassTree::enqueue(std::vector<PlayerNode*>& q, PlayerNode* n) {
  q.push_back(n);
  push_heap(q.begin(),q.end());
}

/** This method removes "orphans", ie players that has no parent and is
 * not root player. The closest (non orphan) player to these orphans is
 * assigned as its parent. After this method is called there will be no
 * orphans left.
 */
void PassTree::removeOrphans() {
  std::vector<PlayerNode>::iterator it;
  std::vector<PlayerNode>::iterator it2;
  int newParentIndex;
  float closestDistance;
  float distance;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    if(it->parentIndex == it->index) { // If orphan...
      closestDistance = 1000;
      newParentIndex = -1;
      // find closest non-orphan player
      for(it2 = allNodes.begin(); it2 != allNodes.end(); it2++) {
	if((it->index == it2->index) // If he is me...
	   || (it2->index == it2->parentIndex)) // ...or if he is an orphan...
	  continue;
	
	distance = (it2->player->getPosition() - 
		    it->player->getPosition()).getLength();
	if(distance < closestDistance) {
	  closestDistance = distance;
	  newParentIndex = it2->index;
	}
      }
      it->parentIndex = newParentIndex;
    }
  }
}

/** Evaluate a position (of the agent) compared to the current position.
 * The position is evaluated considered to the whole team (and the opponents).
 * @return a value between -1 and 1. A negative value means
 * worse position, and a positive means a better position.
 */
float PassTree::evaluatePosition(const TVector& position) {
  if(agentIndex == -1)
    return 0;

  float retVal = getTeamValue(); // also checks the dirty flag
  
  // Save copies of things that will be destroyed
  TVector agentPosition(tm[agentIndex].getPosition());
  
  // Change agent position, recompute pass values and rebuild tree
  tm[agentIndex].setPosition(position);
  build(); // Checks dirty flag and builds pass values and tree

  // the difference between the new and the current team value
  retVal = teamFn() - retVal;

  // Restore
  tm[agentIndex].setPosition(agentPosition);
  dirty = dAgentPassesAndTree;

  return retVal;
}

/** Returns the pass tree in form of PassWay objects with the uniform numbers
 * of the players in the pass tree. If the uniform number of a player
 * is not known, then that pass way is skipped. These pass ways
 * can be used to promote another pass tree to make some hysteresis.
 * The tree is built if needed.
 */
std::vector<PassTree::PassWay> PassTree::getPassWays() {
  if(!isBuilt())
    build();
  return getPassWaysConst();
}

/** Returns the pass tree in form of PassWay objects with the uniform numbers
 * of the players in the pass tree. If the uniform number of a player
 * is not known, then that pass way is skipped. These pass ways
 * can be used to promote another pass tree to make some hysteresis.
 * If the tree needs to be built then an empty vector is returned.
 */
std::vector<PassTree::PassWay> PassTree::getPassWays() const {
  return getPassWaysConst();
}

/** A helper method that returns the passways. If the tree is not
 * built then an empty vector is returned.
 */
std::vector<PassTree::PassWay> PassTree::getPassWaysConst() const {
  std::vector<PassTree::PassWay> passWays;

  if(!isBuilt())
    return passWays; 
  
  std::vector<PlayerNode>::const_iterator it;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    if(it->parentIndex != -1) // is root
      continue;

    PassWay passWay(tm[it->parentIndex].getUniformNumber(),
		    it->player->getUniformNumber());
    if(passWay.isValid())
      passWays.push_back(passWay);
  }
  return passWays;
}

/** Returns an Analysis object that analyses the tree and gives more
 * information about the agent's function in the pass tree.
 * The tree is built if needed. If an analysis object is not in
 * the cache, it will be put there.
 */
PassTree::Analysis PassTree::getAnalysis() {
  if(analysisCache != NULL)
    return *analysisCache;

  if(!isBuilt())
    build();
  analysisCache = new Analysis(*this);
  return *analysisCache;
}

/** Returns an Analysis object that analyses the tree and gives more
 * information about the agent's function in the pass tree.
 * If the tree needs to be built and the cache is empty then a dummy
 * object is returned. <br>
 * <b>Hint:</b> If the non-const method is used first to build the cache then
 * the const method will be less costly (since it otherwise will
 * build a new Analysis object every time this method is called).
 * It will also avoid getting dummy objects when the tree needs to
 * be rebuilt.
 */
PassTree::Analysis PassTree::getAnalysis() const {
  if(analysisCache != NULL)
    return *analysisCache;

  if(!isBuilt())
    return Analysis(); // Dummy object

  return Analysis(*this); 
}

// -------------------
// Heuristic functions
// -------------------


/** Computes a value between 0 and 1 that represents how good a player
 * is placed on the field. Up in the field is good. Scoring position
 * is very good.
 */
float PassTree::fieldFn(const int playerIndex) const {
  const TVector goalL(52.5, -7.01);
  const TVector goalR(52.5, 7.01);
  TVector position(allNodes[playerIndex].player->getPosition());
  
  // If outside field, return zero
  if(position.getX() <= -52.5 || position.getX() >= 52.5 ||
     position.getY() <= -34 || position.getY() >= 34)
    return 0;

  const float minX(-45.0F);
  const float maxX(45.0F);
  float retVal = minMax(0.0F,
			float(0.5 * (position.getX() - minX) /
			      (maxX - minX)),
			0.5F);

  if(position.getX() > 30) {
    float angleL = (goalL-position).getAngle();
    while(angleL >= 360)
      angleL -= 360;
    while(angleL < 0)
      angleL += 360;
    
    float angleR = (goalR-position).getAngle();
    while(angleR >= 360)
      angleR -= 360;
    while(angleR <= angleL)
      angleR += 360;
    
    float angle;
    float minAngleL = 60;
    float minAngleR = 60;
    std::vector<const PlayerState>::iterator it;
    for(it = op.begin(); it != op.end(); it++) {
      angle = (it->getPosition() - position).getAngle();
      while(angle >= 360)
	angle -= 360;
      while(angle <= angleL)
	angle += 360;
      if(angle < angleR) {
	minAngleL = min(minAngleL, angle - angleL);
	minAngleR = min(minAngleR, angleR - angle);
      }
    }
    retVal += max(minAngleL, minAngleR) / 45 * 0.5;
  }
  return retVal;
}

/** Computes a value between 0 and 1 that represents how good a
 * pass is, if only the distance is considered. Very short and very far
 * passes are bad (the rest is good).
 */
float PassTree::distFn(const TVector& from, const TVector& to) const {
  float dist = (to - from).getLength();

  // linear interpolation between these points:
  // (0,0),(minGoodDist,1),(maxGoodDist,1),(maxDist,0),(inf,0)
  const float minGoodDist = 2;
  const float maxGoodDist = 20;
  const float maxDist = 35;

  if(dist < minGoodDist) 
    return dist/minGoodDist;
  
  if(dist < maxGoodDist)
    return 1.0;

  if(dist < maxDist)
    return 1.0 - (dist - maxGoodDist)/(maxDist - maxGoodDist);

  return 0.0;
}

/** Computes a value between 0 and 1 that represents how good the agent
 * avoids the teammates (to avoid a crowd).
 */
float PassTree::avoidFn() const {
  if(agentIndex == -1)
    return 0;
  const float border = 5;
  const float minDist = 4;
  const float maxDist = 35;
  TVector position(tm[agentIndex].getPosition());
  float retValue = 1;
  float value = 1;

  if(position.getX() <= -52.5 || position.getX() >= 52.5 ||
     position.getY() <= -34 || position.getY() >= 34)
    return 0;

  std::vector<const PlayerNode>::iterator it;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    if(it->index == agentIndex || it->index == 1) // don't avoid goalie
      continue;
    
    value = (it->player->getPosition() - position).getLength() / minDist;
    if(value <= 1)
      value = 0;
    else if(value > maxDist/minDist)
      continue;
    else
      value = 1 - 1 / (value*value);
    
    if(value < retValue)
      retValue = value;
  }

  if(position.getX() <= -(52.5 - border) || position.getX() >= (52.5 - border) ||
     position.getY() <= -(34 - border) || position.getY() >= (34 - border))
    retValue *= 0.5;

  return retValue;
}

/** Computes a value between 0 and 1 that represents how good a pass is,
 * if only the opponents are considered.
 * 1 is a good pass (not blocked) and 0 is a bad pass (blocked).
 */
float PassTree::opponentFn(const TVector& from, const TVector& to) const {
  const TVector passVec(to - from);
  
  const float b_max(2.2); // fastest avg ball speed
  const float b_min(1.2); // slowest avg ball speed
  const float action_dist(0.9); // opponent movement
  const float action_penalty(2); // opponent reaction penalty
  const float op_kick_area(1.5); // kickable area of an opponent
  
  float passLen = passVec.getLength();
  if(passLen == 0)
    return 0; // from and to is same position
  
  float retValue(1);
  
  TVector toOp;
  float ballCutOffDist; // distance from from-plr to intercept point
  float opCutOffDist; // distance from op to intercept point
  float actionRadiusMin; 
  float actionRadiusMax;
  // Check opponents
  std::vector<PlayerState>::const_iterator it;
  for(it = op.begin(); it != op.end(); it++) {
    toOp = it->getPosition() - from;
    ballCutOffDist = toOp * passVec / passLen;
    if(ballCutOffDist < 0 ||
       ballCutOffDist > passLen + 2) 
      continue;
    
    actionRadiusMin = (ballCutOffDist / b_max - action_penalty) * action_dist;
    actionRadiusMax = (ballCutOffDist / b_min - action_penalty) * action_dist;

    if(actionRadiusMin < 0)
      actionRadiusMin = 0;
    if(actionRadiusMax < 0)
      actionRadiusMax = 0;
    
    actionRadiusMin += op_kick_area;
    actionRadiusMax += op_kick_area;
    
    opCutOffDist = (toOp - passVec * (ballCutOffDist / passLen)).getLength();
    
    if(opCutOffDist < actionRadiusMin)
      return 0;

    if(opCutOffDist < actionRadiusMax) {
      retValue = min(retValue,
		     1.0F -
		     (actionRadiusMax - opCutOffDist) / 
		     (actionRadiusMax - actionRadiusMin));
    }
  }
  
  return retValue;
}

/** Computes a value between 0 and 1 that represents how good the
 * agent and the team is placed. It is good if the agent avoids the 
 * teammates and if the team can get the ball to a scoring position.
 */
float PassTree::teamFn() const {
  const float wTeam_spread = 0.2;
  const float wTeam_score = 0.8;
  
  // A high value (greater tha 1) of wField will make the field value 
  // more important than the get ball value
  const float wField(2.0); 
  
  float scoreValue = 0;
  float spreadValue = 0;

  // compute spread-value
  spreadValue = avoidFn();
  if(spreadValue < 0)
    spreadValue = 0;
  else if(spreadValue > 1)
    spreadValue = 1;

  // compute score-value
  std::vector<const PlayerNode>::iterator it;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    if(it->parentIndex != -1) { 
      scoreValue += it->value * pow(fieldFn(it->index), wField);
    }
  }
  scoreValue *= 1.5 / (allNodes.size() + 0.00001);
  if(scoreValue < 0)
    scoreValue = 0;
  else if(scoreValue > 1)
    scoreValue = 1;

  return wTeam_score * scoreValue + wTeam_spread * spreadValue;
}

/** Displays the pass tree. Draws arrows on all pass ways and circles around
 * the root players. If the tree needs to be built then it will be.
 */
void PassTree::display() {
  if(!isBuilt())
    build();
  Display* display(Display::instance());
  std::vector<PlayerNode>::iterator it;
  for(it = allNodes.begin(); it != allNodes.end(); it++) {
    if(it->parentIndex == -1) { // root player
      display->addCircle(it->player->getPosition(),
			 1.0,
			 1,
			 Display::cRed);
    } else { // non-root player
      display->addArrow(tm[it->parentIndex].getPosition(),
			it->player->getPosition(),
			1,
			Display::cBlack);
    }
  }
}

void PassTree::printPassValues() {
  cout << "un = ";
  for(unsigned int i=0; i < tm.size(); i++) 
    if(tm[i].getUniformNumber() != -1)
      cout << tm[i].getUniformNumber() << ", ";
  cout << endl;

  cout << "pr = ";
  for(unsigned int i=0; i < promotedPassValues.size(); i++) 
    for(unsigned int j=0; j < promotedPassValues.size(); j++) 
      if(promotedPassValues[i][j] > 0) 
	cout << tm[i].getUniformNumber() << "->" 
	     << tm[j].getUniformNumber() << ":" 
	     << promotedPassValues[i][j] << ", ";
  cout << endl;

  cout << "pv = ";
  for(unsigned int i=0; i < passValues.size(); i++) 
    for(unsigned int j=0; j < passValues.size(); j++) 
      if(passValues[i][j] > 0) 
	cout << tm[i].getUniformNumber() << "->" 
	     << tm[j].getUniformNumber() << ":" 
	     << passValues[i][j] << ", ";
  cout << endl;
}

// ----------
// PlayerNode
// ----------

PassTree::PlayerNode::PlayerNode(PlayerState* _player,
				 int _index) 
  : index(_index),
    parentIndex(_index),
    player(_player),
    value(0)
{
}

PassTree::PlayerNode::~PlayerNode() {
}

PassTree::PlayerNode& PassTree::PlayerNode::operator=(const PlayerNode& obj) {
  player = obj.player;
  value = obj.value;
  index = obj.index;
  parentIndex = obj.parentIndex;
  return *this;
}


// -------
// PassWay
// -------

/** Dummy constructor. Constructs an invalid pass way. 
 * This constructor exists so that empty objects can be created.
 */
PassTree::PassWay::PassWay() 
  : from(0),
    to(0)
{
}

/** Constructs a pass way. The passway is from fromUniformNumber to
 * toUniformNumber. The isValid() method can be used to check if the
 * uniform numbers are valid.
 * @see isValid()
 */
PassTree::PassWay::PassWay(int fromUniformNumber, int toUniformNumber) 
  : from(fromUniformNumber),
    to(toUniformNumber)
{
}

/** Checks if the uniform numbers are valid (between 1 and 11) and
 * not the same.
 */
bool PassTree::PassWay::isValid() const {
  if(1 <= from && from <= 11 &&
     1 <= to && to <= 11 &&
     to != from)
    return true;

  return false;
}

// --------
// Analysis
// --------

PassTree::Analysis::Analysis() 
{
}

PassTree::Analysis::Analysis(const PassTree& pt)
  : treePosition(tpNotInScoreBranch),
    depthToRoot(0),
    depthToScore(0),
    numChildren(0),
    passValueToTarget(0),
    agentScoreValue(0),
    bestScoreValue(0)
{
  if(pt.agentIndex == -1)
    return;

  const float wScore_getBall(0.3);
  const float wScore_field(0.7);

  int bestScoreIndex = 0;
  float value;
  std::vector<PlayerNode>::const_iterator it;
  for(it = pt.allNodes.begin(); it != pt.allNodes.end(); it++) {
    value =  wScore_getBall * it->value 
      + wScore_field * pt.fieldFn(it->index);
    if(value > bestScoreValue) {
      bestScoreValue = value;
      bestScoreIndex = it->index;
    }
    if(it->index == pt.agentIndex)
      agentScoreValue = value;
    if(it->parentIndex == pt.agentIndex)
      numChildren++;
  }
  
  // count depth to root and score
  bool foundScorePlayer(false);
  int parentIndex = pt.allNodes[pt.agentIndex].parentIndex;
  while(parentIndex != -1) {
    if(parentIndex == bestScoreIndex) {
      foundScorePlayer = true;
      depthToScore = depthToRoot + 1;
    }
    parentIndex = pt.allNodes[parentIndex].parentIndex;
    depthToRoot++;
  }

  // count depth from score to agent
  bool foundAgent(false);
  int depthToAgent(0);
  int passIndex(0);
  if(!foundScorePlayer && numChildren > 0) {
    parentIndex = pt.allNodes[bestScoreIndex].parentIndex;
    while(parentIndex != -1) {
      if(parentIndex == pt.agentIndex) {
	foundAgent = true;
	break;
      }
      passIndex = parentIndex;
      parentIndex = pt.allNodes[parentIndex].parentIndex;
      depthToAgent++;
    }
  }

  // find pass target
  if(numChildren > 0 && foundAgent) {
    if(passIndex == pt.agentIndex)
      passIndex = bestScoreIndex;
    passTarget = pt.tm[passIndex];
    passValueToTarget = pt.passValues[pt.agentIndex][passIndex];
  }

  // figure out tree position
  if(bestScoreIndex == pt.agentIndex) { 
    if(pt.allNodes[pt.agentIndex].parentIndex == -1)
      treePosition = tpScoreAndRootPlayer;
    else
      treePosition = tpScorePlayer;
  } else {
    if(pt.allNodes[pt.agentIndex].parentIndex == -1)
      treePosition = tpRootPlayer;
    else { // neither score or root
      if(foundScorePlayer)
	treePosition = tpAfterScorePlayer;
      else if(foundAgent)
	treePosition = tpBeforeScorePlayer;
      else
	treePosition = tpNotInScoreBranch; 
    }
  }
}

bool PassTree::Analysis::isRoot() const {
  return (treePosition == tpRootPlayer) ||
    (treePosition == tpScoreAndRootPlayer);
}

bool PassTree::Analysis::inScoreBranch() const {
  return treePosition != tpNotInScoreBranch;
}

bool PassTree::Analysis::hasPassTarget() const {
  return passValueToTarget != 0;
}

RS_END_NAMESPACE


/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler or debugger does not understand the keyword inline then
 * include the inline definitions here, otherwise include them in the
 * declaration file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifndef RS_USE_INLINE
#  include "PassTree.icc"
#endif



