/* -*- Mode: C++ -*-
 * $Id: PassTree.h,v 1.30 2001/07/30 17:00:01 mikbr212 Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: PassTree.h
 * Unitname: Framework/view
 * $Revision: 1.30 $
 * Last modified by $Author: mikbr212 $ $Date: 2001/07/30 17:00:01 $
 *
 *
 * HISTORY
 *
 * $Log: PassTree.h,v $
 * Revision 1.30  2001/07/30 17:00:01  mikbr212
 * Debugged and fixed stupid static bug
 *
 * Revision 1.29  2001/07/30 15:42:19  mikbr212
 * Added comments
 *
 * Revision 1.28  2001/07/30 14:58:52  mikbr212
 * Added setStatic option.
 *
 * Revision 1.27  2001/07/27 14:36:05  mikbr212
 * Added score values in analysis
 *
 * Revision 1.26  2001/07/26 13:15:32  mikbr212
 * Removed some old stuff (old public methods)
 *
 * Revision 1.25  2001/07/26 12:20:46  mikbr212
 * Changed passtree completly
 *
 * Revision 1.24  2001/07/06 10:00:38  mikbr212
 * Merged updates and resolved conflicts (a lot)
 *
 * Revision 1.23  2001/06/19 14:01:00  mikbr212
 * Fixed circular includes stuff. Somebody should check that I did it right.
 *
 * Revision 1.22  2001/06/05 20:33:53  mikbr212
 * Fixed doxygen-stubs for all classes
 *
 * Revision 1.21  2001/04/24 21:44:05  mikbr212
 * adjusted weights
 *
 * Revision 1.20  2001/04/24 21:27:41  mikbr212
 * ??
 *
 * Revision 1.19  2001/04/24 17:10:00  mikbr212
 * updated a lot
 *
 * Revision 1.18  2001/04/24 13:51:27  mikbr212
 * added getPassValueToPassTarget
 *
 * Revision 1.17  2001/04/24 12:36:59  mikbr212
 * fixed stuff
 *
 * Revision 1.16  2001/04/23 22:49:25  mikbr212
 * added some methods: getGoodGuys etc
 *
 * Revision 1.15  2001/04/23 15:51:29  mikbr212
 * added getTeamValue()
 *
 * Revision 1.14  2001/04/23 13:12:18  mikbr212
 * added getAgentIndex
 *
 * Revision 1.13  2001/04/23 13:10:44  mikbr212
 * added scoreValues
 *
 * Revision 1.12  2001/04/23 10:44:48  mikbr212
 * fixed getPosition
 *
 * Revision 1.11  2001/04/22 14:11:26  mikbr212
 * fixed offside for rightSide-players also
 *
 * Revision 1.10  2001/04/22 12:15:54  mikbr212
 * added offside-stuff
 *
 * Revision 1.9  2001/04/21 16:26:52  mikbr212
 * fixed stuff
 *
 * Revision 1.8  2001/04/20 18:56:04  mikbr212
 * ???
 *
 * Revision 1.7  2001/04/20 12:59:13  mikbr212
 * fixed copy and assignment
 *
 * Revision 1.6  2001/04/20 12:03:41  mikbr212
 * added display
 *
 * Revision 1.5  2001/04/20 11:40:33  sigge
 * A bunch of const fixes.
 *
 * Revision 1.4  2001/04/20 09:42:02  mikbr212
 * added const and dummy-constructor
 *
 * Revision 1.3  2001/04/20 09:30:50  mikbr212
 * added copy-constructor and assignment-operator
 *
 * Revision 1.2  2001/04/19 22:25:45  mikbr212
 * Added some methods
 *
 * Revision 1.1  2001/04/19 22:16:28  mikbr212
 * added PassTree
 *
 *
 */

/**
 * @file PassTree.h
 * Include file for the view class PassTree.
 *
 * @par Unit:
 *      Framework/view
 * $Revision: 1.30 $
 * $Author: mikbr212 $
 * $Date: 2001/07/30 17:00:01 $
 */


#ifndef PASSTREE_H
#define PASSTREE_H


#include "TLine.h"
#include "TVector.h"
#include "WorldState.h"
//  #include "Display.h"
#include <vector>
#include <algorithm>

RS_BEGIN_NAMESPACE

class Display;

/**
 * @class PassTree PassTree.h
 * This class builds a pass tree between all teammates.
 * Pass values are computed between all teammates, both
 * distance between the players and distance to opponents
 * is considered.
 * Then a tree is built using Dijkstra, containing the best
 * pass ways. The player closest to the ball is considered
 * as the root node. <br>
 * The pass tree can be used to evaluate different positions
 * to get more passable, and to get the best pass target etc
 * (from the Analysis object).
 * 
 * @par Unit:
 *      Framework/view
 * $Revision: 1.30 $
 * $Author: mikbr212 $
 * $Date: 2001/07/30 17:00:01 $
 */


class PassTree {
public:
  // --------------
  // Nested classes
  // --------------
protected:
  /** Enum describing what things in the tree that needs to be
   * rebuilt.
   * dClean = nothing.
   * dAgentPassesAndTree = only the passes to and from the current agent.
   * dAllPassesAndTree = all pass values.
   * dTreeOnly = only the pass tree.
   */
  enum Dirty {
    dClean,
    dAgentPassesAndTree,
    dAllPassesAndTree,
    dTreeOnly
  };

  /**
   * @class PlayerNode PassTree.h
   * Text...
   * $Revision: 1.30 $
   * $Author: mikbr212 $
   * $Date: 2001/07/30 17:00:01 $
   */
  // PlayerNode
  class PlayerNode {
  public:
    PlayerNode(PlayerState* _player,
	       int _index);
    
    ~PlayerNode();
    
    PlayerNode& operator=(const PlayerNode& obj);
    bool operator<(const PlayerNode& obj);
    
    int index;
    int parentIndex;
    PlayerState* player;
    float value;
  };

public:
  /**
   * @class Analysis PassTree.h
   * The analysis object is used to analyse the pass tree
   * and to find interesting data from it. 
   * 
   * $Revision: 1.30 $
   * $Author: mikbr212 $
   * $Date: 2001/07/30 17:00:01 $
   */
  class Analysis { // kolla lite mer hr sen
    // Nested types
  public:
    enum TreePosition {
      tpNotInScoreBranch,
      tpRootPlayer,
      tpScorePlayer,
      tpScoreAndRootPlayer,
      tpBeforeScorePlayer,
      tpAfterScorePlayer
    };
    
    // Methods and attributes
  public:
    Analysis();
    TreePosition getTreePosition() const;
    int getDepthToRootPlayer() const;
    int getDepthToScorePlayer() const;
    int getNumberOfChildren() const;
    const PlayerState& getPassTarget() const; 
    float getPassValueToTarget() const;
    float getAgentScoreValue() const;
    float getBestScoreValue() const;

    // shortcuts
    bool isRoot() const;
    bool inScoreBranch() const;
    bool hasPassTarget() const;

  protected:
    friend PassTree; // Allow PassTree to use this constructor
    Analysis(const PassTree& pt);
  private:
    TreePosition treePosition;
    int depthToRoot;
    int depthToScore;
    int numChildren;
    PlayerState passTarget;
    float passValueToTarget;
    float agentScoreValue;
    float bestScoreValue;
  };

  /**
   * @class PassWay PassTree.h
   * This class encapsules a pass way from a player
   * to another. The uniform number must be known of both players.
   * Actually this class simply is a pair of uniform numbers.
   *
   * $Revision: 1.30 $
   * $Author: mikbr212 $
   * $Date: 2001/07/30 17:00:01 $
   */
  // PassWay
  class PassWay {
  public:
    PassWay(); 
    PassWay(int, int);
    int getFromUniformNumber() const;
    int getToUniformNumber() const;
    bool isValid() const;
  protected:
    int from;
    int to;
  };
  
  // ----------------------
  // Methods and attributes
  // ----------------------
public:
  friend Analysis; // allow Analysis to analyse me
  PassTree(); // Dummy constructor

  PassTree(const WorldState& world,
	   bool addAgent = true);

  PassTree(const PassTree& other);
  PassTree& operator= (const PassTree& other);  

  ~PassTree();

  void display();
  void printPassValues();

  void promotePassWay(const PassTree::PassWay& passWay, float value);
  void promotePassWays(const std::vector<PassTree::PassWay>& passWays, 
		       float value);

  void setStatic(bool);

  bool isBuilt() const;

  void build();
  void build(int _agentIndex);

  float evaluatePosition(const TVector& position); 
    
  const std::vector<PlayerState>& getTeammates() const;
  const std::vector<PlayerState>& getOpponents() const;

  float getTeamValue();
  float getTeamValue() const;
  
  PassTree::Analysis getAnalysis();
  PassTree::Analysis getAnalysis() const;

  std::vector<PassTree::PassWay> getPassWays();
  std::vector<PassTree::PassWay> getPassWays() const;
protected:
  std::vector<PassTree::PassWay> getPassWaysConst() const;

  // Heuristic functions
  float distFn(const TVector& from, 
	       const TVector& to) const;
  float opponentFn(const TVector& from, 
		   const TVector& to) const;

  float fieldFn(const int playerIndex) const;
  // Combined heuristic functions
  float avoidFn() const;
  float teamFn() const;

  // build-tree functions
  void buildPassValues();
  void rebuildPassValues(int playerIndex);
  void initNodes(const BallState& ball);
  void buildTree();
  void removeOrphans();

  PlayerNode* dequeue(std::vector<PlayerNode*>& q);
  void enqueue(std::vector<PlayerNode*>& q, PlayerNode* n);

  // Attributes
  /** Dirty flag that tells if something needs to be rebuilt. */
  Dirty dirty;
  /** The current agent. Index in tm vector. -1 represents no agent. */
  int agentIndex;
  /** Vector of the teammates. */
  std::vector<PlayerState> tm;
  /** Vector of the opponents. */
  std::vector<PlayerState> op; 
  
  /** Vector of all the nodes in the tree. Has pointers to the tm vector. */
  std::vector<PlayerNode> allNodes;
  /** Contains pass values that are promoted. */
  std::vector<std::vector<float> > promotedPassValues;
  /** Contains the pass values. */
  std::vector<std::vector<float> > passValues;
  /** Contains the team value for the for the first built tree. */
  float teamValue;
  /** Contains a cached value of the Analysis object. */
  Analysis* analysisCache;
  /** Tells whether the tree is static or dynamic. */
  bool staticTree;
  /** Contains translations from uniform numbers to indices. */
  std::vector<int> indices;
  /** True if indices is built. */
  bool indicesBuilt;
};


RS_END_NAMESPACE

/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "PassTree.icc"
#endif

#endif // !PASSTREE_H
