/* -*- Mode: C++ -*-
 * $Id: FuzzyPlayer.h,v 1.1.1.1 2001/04/08 11:33:59 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: FuzzyPlayer.h
 * Unitname: Library
 * $Revision: 1.1.1.1 $
 * Created by: Mikael Brnnstrm
 * Last modified by $Author: sigge $
 *
 *
 * HISTORY
 *
 * $Log: FuzzyPlayer.h,v $
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 */

/**
 * @file FuzzyPlayer.h
 * Definition of the class FuzzyPlayer.
 *
 * @par Unit:
 *      Library
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */

#ifndef FUZZY_PLAYER_H
#define FUZZY_PLAYER_H

//#include "CoordVectorUCO.h"
//#include "CoordPointUCO.h"
#include "library_types_with_unknown.h"
#include "machine_dependent_types.h"
#include "FuzzyBool.h"

/*
  TODO:
*/

RS_BEGIN_NAMESPACE

enum TeamBinary {
  TB_OUR_TEAM,
  TB_THEIR_TEAM
};

typedef WithUCO<TeamBinary,TimeUnknown> TeamUCO;

typedef WithUCO<UInt16,TimeUnknown> UInt16UCO;

class FuzzyPlayer {
public:
  FuzzyPlayer(); 
  /*PointUCO _pos = PointUCO(FloatUCO(0,Time(),Confidence(0)),
    FloatUCO(0,Time(),Confidence(0))),
    VectorUCO _speed = VectorUCO(AngleDegUCO(0,Time(),Confidence(0)),
    FloatUCO(0,Time(),Confidence(0))),
    TeamUCO _team = TeamUCO(TB_OUR_TEAM,Time(),Confidence(0)),
    UInt16UCO _uNum = UInt16UCO(0,Time(),Confidence(0));
    AngleDegUCO _bodyDir = AngleDegUCO(0,Time(),Confidence(0)),
    AngleDegUCO _neckDir = AngleDegUCO(0,Time(),Confidence(0))
    ); */
  
  FuzzyPlayer(const FuzzyPlayer& p1, const FuzzyPlayer& p2);
  virtual ~FuzzyPlayer();
  
  PointUCO getPos() const;
  void setPos(const PointUCO& p);
  
  VectorUCO getSpeed() const;
  void setSpeed(const VectorUCO& v);
  
  UInt16UCO getUNum() const;
  void setUNum(const UInt16UCO& i);

  AngleDegUCO getNeckDir() const;
  void setNeckDir(const AngleDegUCO& a);

  AngleDegUCO getBodyDir() const;
  void setBodyDir(const AngleDegUCO& a);

  TeamUCO getTeam() const;
  void setTeam(const TeamUCO& t);
  
  virtual void predict();
  virtual void degrade(float f = 0.9);
  Confidence getConfidence() const;

  FuzzyBool isMatch(const FuzzyPlayer& plr);
  //  FuzzyPlayer union(const FuzzyPlayer& plr);

  void fixUnknown();

protected:
  void clearConfidence();

  void adjustForConfidence(Float& f,
			   const Confidence& c1,
			   const Confidence& c2) const;
  
  Confidence FuzzyPlayer::vectorConfidence(const VectorUCO& vec) const;
  Confidence FuzzyPlayer::pointConfidence(const PointUCO& p) const;
  
  PointUCO pos;
  VectorUCO speed;
  UInt16UCO uNum;
  AngleDegUCO neckDir;
  AngleDegUCO bodyDir;
  TeamUCO team;
};

RS_END_NAMESPACE

#endif // !FUZZY_PLAYER


