/* -*- Mode: C++ -*-
 * $Id: Stub.h,v 1.1.1.1 2001/04/08 11:33:59 sigge Exp $
 *
 *
 * COPYRIGHT INFORMATION
 *
 * This file is part of RoboSoc created by Fredrik Heintz <frehe@ida.liu.se>
 * Copyright (C) 1999, 2000 Fredrik Heintz, Linkping University, Sweden
 *
 * You are allowed to modify and use this code as long as you retain this
 * notice. If you make any changes or have any comments I would appreciate
 * it if you send me a message. For more information, please see
 * http://www.ida.liu.se/~frehe/RoboCup/RoboSoc/
 *
 *
 * IDENTIFICATION
 *
 * Filename: Stub.h
 * Unitname: Library | BasicSystem | Framework | RoboSoc
 * $Revision: 1.1.1.1 $
 * Created by: Fredrik Heintz 2000-08-19
 * Last modified by $Author: sigge $ $Date: 2001/04/08 11:33:59 $
 *
 *
 * HISTORY
 *
 * $Log: Stub.h,v $
 * Revision 1.1.1.1  2001/04/08 11:33:59  sigge
 * Imported sources
 *
 * Revision 1.1.1.1  2000/08/31 13:26:03  frehe
 * Importet the current version of RoboSoc (soon to be v2.5.0)
 *
 * Revision 1.1  2000/08/22 13:33:56  robocup
 * Made many minor changes. Started to do the complete refactoring and documentation effort. [frehe]
 *
 * Revision 1.4  2000/08/22 11:58:55  robocup
 * Made many minor modifications to the stub for header files. [frehe]
 *
 * Revision 1.3  2000/08/21 12:06:42  robocup
 * Updated the stub for header files with Doxygen tags. [frehe]
 *
 * Revision 1.2  2000/08/21 08:20:31  robocup
 * Made minor changes to the stub for header files. [frehe]
 *
 * Revision 1.1  2000/08/21 08:11:01  robocup
 * Added a stub for header files. [frehe]
 */

/**
 * @file Stub.h
 * This is a stub for header files that all RoboSoc header files should use.
 * A header file contains all the declarations needed by a class or a unit.
 *
 * @par Unit:
 *      Library | BasicSystem | Framework | RoboSoc
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */


#ifndef STUB_H
#define STUB_H


//#ifdef HAVE_CONFIG_H
//#  ifndef CONFIG_H
//#    include "config.h"
//#  endif
//#endif

//#include <...>
//#include <...>

#include "machine_dependent_compiler_directives.h"
//#include "library_compiler_directives.h"
//#include "basic_system_compiler_directives.h"
//#include "framework_compiler_directives.h"
//#include "SuperClass.h"
//#include "TheType.h"


RS_BEGIN_NAMESPACE


/**
 * @class Stub Stub.h
 * The first sentence is a brief description of the class Stub.
 * The rest is a longer description of the class Stub.
 *
 * @pre The preconditions for constructing the class are ...
 * @post The postconditions after constructing the class are...
 * @invariant The class invariant is...
 *
 * @todo Things that should be done...
 *
 * @par Unit:
 *      Library | BasicSystem | Framework | RoboSoc
 * $Revision: 1.1.1.1 $
 * $Author: sigge $
 * $Date: 2001/04/08 11:33:59 $
 */
class Stub : public SuperClass {
public:
  /** The default constructor.
    @pre The preconditions are ...
    @post The postconditions are... */
  Stub();

  /** The copy constructor.
    @param obj is the object to be copied.
    @post The constructed object is an exact copy of @a obj. */
  Stub(const Stub& obj);

  /** The destructor. */
  virtual ~Stub();

  
  /** The assignment operator.
    @param obj is the object to be assigned from.
    @return A reference to the the newly assigned object.
    @post The object is an exact copy of @a obj. */
  Stub& operator=(const Stub& obj);


  /** Test the class invariant.
    This function implements a test of the class invariant.
    It should be tested after each constructor, before each destructor,
    and at the end of methods that change the state of the object.
    @return @c true iff the class invariant is satisfied, otherwise @c false. */
  bool invariant() const;
  
    
  /** Description of this method.
    @param arg used to do something.
    @exception MyException is thrown if something goes wrong.
    @return A TheType as the result of doing something.
    @pre The preconditions are ...
    @post The postconditions are... */
  TheType methodName(const TheType const* arg) const throw(MyException);

  
protected:
  /** Description of this accessor to the attribute.
    @return A description of the return value.
    @pre The preconditions are ...
    @post The postconditions are... */
  TheType getAttributValue() const;

  /** Description of this modifier of the attribute.
   @param new_value a description of the parameter.
   @pre The preconditions are ...
   @post The postconditions are... */  
  void setAttributValue(TheType new_value);

  
private:
  /** Description of this attribute. */
  TheType attributeName;

  /** Description of this attribute. */
  TheType* attributePointer;
};


RS_END_NAMESPACE

  
/* CONDITIONAL INCLUSION OF INLINE DEFINITIONS
 *
 * If the compiler and debugger understands the keyword inline then include
 * the inline definitions here, otherwise include them in the definition file.
 *
 * This is controlled by a flag called RS_USE_INLINE, which is usually
 * defined. It can be set by the user by giving the compiler an argument
 * usually -DRS_USE_INLINE (to define it) or -URS_USE_INLINE (to undefine it).
 */

#ifdef RS_USE_INLINE
#  include "Stub.icc"
#endif


#endif // !STUB_H
