package robosoc;

import java.io.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;
import java.util.*;

public class Monitor extends JFrame implements Runnable
{
    /**
     */
    public static void main(String[] args) {
	Monitor m = new Monitor(args);
    }

    private SoccerField soccerField;
    private Thread inputThread;
    private InputStream in;
    private StatusView statusView;
    private JTextArea messages;

    private boolean fileMode = false;

    /**
     */
    Monitor(String[] args) {
	super("RoboSoc Client Monitor");

	// read args
	String filename = "";
	boolean createFifo = false;
	for(int i=1; i<args.length; i++) {
	    if(args[i-1].equals("-file")) {
		filename = args[i++];
		fileMode = true;
	    } else if(args[i-1].equals("-player")) {
	      String userName = java.lang.System.getProperty("user.name");
		filename = "/tmp/rs_monitor"+args[i++]+"_" + userName + ".txt";
		createFifo = true;
	    }
	}

	if(createFifo) {
	    try {
		// delete file/fifo if it exists
		File file = new File(filename);
		if(file.exists())
		    file.delete();
		// (try to) create fifo
		Process mkfifo = Runtime.getRuntime().exec("mkfifo "+filename);
		if(mkfifo.waitFor() != 0) {
		    System.err.println("Could not create fifo:");
		    BufferedReader reader = new BufferedReader(new InputStreamReader(mkfifo.getErrorStream()));
		    while(reader.ready())
			System.out.println(reader.readLine());
		    System.exit(1);
		}
	    } catch (InterruptedException e) {
		System.err.println("Interrupted!");
		System.exit(1);
	    } catch (IOException e) {
		System.err.println("Could not create fifo: "+e);
		System.exit(1);
	    }
	}

	if(filename.equals("")) {
	    in = System.in;
	} else {
	    try {
		in = new FileInputStream(filename);
	    } catch (FileNotFoundException e) {
		System.err.println("File not found: "+e);
	    }
	}

	// Listen to windowClosing-event
	this.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    System.exit(0);
		}
	    });


	// create stuff
	soccerField = new SoccerField(4.0);
	soccerField.setSize(200,100);

	statusView = new StatusView();
	messages = new JTextArea("Starting...");
  	messages.setColumns(20);

	JScrollPane msgScroll = new JScrollPane(messages);

	getContentPane().setLayout(new BorderLayout(2,2));
	getContentPane().add(soccerField, BorderLayout.WEST);
	getContentPane().add(statusView, BorderLayout.SOUTH);
	getContentPane().add(msgScroll, BorderLayout.CENTER);

	// fix size
	pack();
	setResizable(true);
	setSize(getPreferredSize());
	show();

	// Start reading
	inputThread = new Thread(this);
	inputThread.setDaemon(true);
	inputThread.run();
    }

    private double fix2double(int fix)
    {
      return ((double)fix)/32768.0 - 1.0;
    }

    private Color getColor(int c)
    {
      Color color;
      switch(c)
      {
	case 0:
	  color = Color.black;
	  break;
	case 1:
	  color = Color.blue;
	  break;
	case 2:
	  color = Color.cyan;
	  break;
	case 3:
	  color = Color.darkGray;
	  break;
	case 4:
	  color = Color.gray;
	  break;
	case 5:
	  color = Color.lightGray;
	  break;
	case 6:
	  color = Color.magenta;
	  break;
	case 7:
	  color = Color.orange;
	  break;
	case 8:
	  color = Color.pink;
	  break;
	case 9:
	  color = Color.red;
	  break;
	case 10:
	  color = Color.white;
	  break;
	case 11:
	  color = Color.yellow;
	  break;
        default:
	  color = Color.white;
	  break;
      }
      return color;
    }

    private int unsign_byte(byte b)
    {
      if(b >= 0)
	return b;
      else
	return b + 256;
    }

    private int read_byte(InputStream in) throws IOException
    {
      byte foo[] = new byte[1];
      in.read(foo);
      return unsign_byte(foo[0]);
    }

    private SoccerField.SoccerBall read_ball(InputStream in) throws IOException
    {
      byte bytes[] = new byte[5];
      int num_bytes = in.read(bytes);
      int xpos, ypos;
      xpos = (unsign_byte(bytes[0]) << 8) + unsign_byte(bytes[1]);
      ypos = (unsign_byte(bytes[2]) << 8) + unsign_byte(bytes[3]);
      int confidence = unsign_byte(bytes[4]);
      return new SoccerField.SoccerBall(fix2double(xpos)*52.5, fix2double(ypos)*34.0,
					fix2double(256*confidence));
    }

    private SoccerField.SoccerPlayer read_player(InputStream in) throws IOException
    {
      byte bytes[] = new byte[7];
      int num_bytes = in.read(bytes);
      int xpos, ypos;
      xpos = (unsign_byte(bytes[0]) << 8) + unsign_byte(bytes[1]);
      ypos = (unsign_byte(bytes[2]) << 8) + unsign_byte(bytes[3]);
      int confidence = unsign_byte(bytes[4]);
      int team = unsign_byte(bytes[5]) & 0x3;
      int no = (unsign_byte(bytes[5]) & 0x3c) >> 2;
      boolean me = (unsign_byte(bytes[5]) & 0x40) == 0x40;
      int bodydir = unsign_byte(bytes[6]);
      return new SoccerField.SoccerPlayer(fix2double(xpos)*52.5, fix2double(ypos)*34.0,
					  fix2double(256*confidence),
					  team, 360.0*fix2double(bodydir), no, (boolean)me);
    }

    private SoccerField.Arrow read_arrow(InputStream in) throws IOException
    {
      byte bytes[] = new byte[10];
      int num_bytes = in.read(bytes);
      int x1, y1, x2, y2;
      x1 = (unsign_byte(bytes[0]) << 8)	+ unsign_byte(bytes[1]);
      y1 = (unsign_byte(bytes[2]) << 8)	+ unsign_byte(bytes[3]);
      x2 = (unsign_byte(bytes[4]) << 8)	+ unsign_byte(bytes[5]);
      y2 = (unsign_byte(bytes[6]) << 8)	+ unsign_byte(bytes[7]);
      int cyclesLeft = unsign_byte(bytes[8]);
      Color color = getColor(unsign_byte(bytes[9]));

      return new SoccerField.Arrow(fix2double(x1)*52.5, fix2double(y1)*34.0,
				   fix2double(x2)*52.5, fix2double(y2)*34.0,
				   color, cyclesLeft);
    }

    private SoccerField.Circle read_circle(InputStream in) throws IOException
    {
      byte bytes[] = new byte[8];
      int num_bytes = in.read(bytes);
      int x1, y1, r;
      x1 = (unsign_byte(bytes[0]) << 8) + unsign_byte(bytes[1]);
      y1 = (unsign_byte(bytes[2]) << 8) + unsign_byte(bytes[3]);
      r = (unsign_byte(bytes[4]) << 8) + unsign_byte(bytes[5]);
      int cyclesLeft = unsign_byte(bytes[6]);
      Color color = getColor(unsign_byte(bytes[7]));

      return new SoccerField.Circle(fix2double(x1)*52.5, fix2double(y1)*34.0,
				    fix2double(r)*100.0, color, cyclesLeft);
    }

    public void run() {
	Vector players = new Vector();
	Vector arrows = new Vector();
	Vector circles = new Vector();
	SoccerField.SoccerBall ball = null;

	try {
	    Object o;
	    //  	    byte[] buffer = new byte[1024];
	    while(true) {
	      players.clear();
	      arrows.clear();
	      circles.clear();
	      ball = null;

	      ball = read_ball(in);

	      int num_players = read_byte(in);
	      for(int i=0; i<num_players; i++)
		players.add(read_player(in));

	      int num_arrows = read_byte(in);
	      for(int i=0; i<num_arrows; i++)
		arrows.add(read_arrow(in));

	      int num_circles = read_byte(in);
	      for(int i=0; i<num_circles; i++)
		circles.add(read_circle(in));

	      soccerField.newCycle();
	      soccerField.setPlayers(players);
	      soccerField.addArrows(arrows);
	      soccerField.addCircles(circles);
	      soccerField.setBall(ball);
	      soccerField.repaint();

	      if(fileMode) {
		try {
		  Thread.currentThread().sleep(50);
		} catch(InterruptedException e) {}
	      }

	    } // while
	} catch(IOException ioe) {
	    System.err.println("IO error: "+ioe.toString());
	    System.exit(0);
	}
    }
}
