package robosoc;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.awt.image.*;
import java.awt.event.*;

public class SoccerField extends JPanel
{
    public static class ParseErrorException extends RuntimeException {
	private String str;
	public ParseErrorException(String s) {
	    str = s;
	}

	public String toString() {
	    return "ParseErrorException: "+str;
	}
    }

    public static class Arrow {
	private double x1;
	private double y1;
	private double x2;
	private double y2;
	private Color color;
	private int cyclesLeft;

	public Arrow(double _x1, double _y1, double _x2, double _y2,
		     Color _color, int _cyclesleft) {
	    x1 = _x1;
	    y1 = _y1;
	    x2 = _x2;
	    y2 = _y2;
	    color = _color;
	    cyclesLeft = _cyclesleft;
	}

	public void draw(Graphics2D g2) {
	    g2.setColor(color);
	    g2.drawLine((int)x1,(int)y1,(int)x2,(int)y2);
	    g2.drawOval((int)x2-1,(int)y2-1,2,2);
	}

	public boolean newCycle() {
	    cyclesLeft--;
	    return cyclesLeft != 0;
	}

    }

    public static class Circle {
	private double x;
	private double y;
	private double r;
	private Color color;
	private int cyclesLeft;

	public Circle(double _x, double _y, double _r, Color _color, int _cyclesleft) {
	    x = _x;
	    y = _y;
	    r = _r;
	    color = _color;
	    cyclesLeft = _cyclesleft;
	}

	public void draw(Graphics2D g2) {
	    g2.setColor(color);
	    g2.drawOval((int)(x-r),(int)(y-r),2*(int)r,2*(int)r);
	}

	public boolean newCycle() {
	    cyclesLeft--;
	    return cyclesLeft != 0;
	}

    }

    public static class SoccerBall {
	private double x;
	private double y;
	private double confidence;


	public SoccerBall(double xpos, double ypos, double c) {

	  x = xpos;
	  y = ypos;
	  confidence = c;

	  // Adjust confidence to appropriate levels, because of fixed
	  // point roundoff errors
	  confidence = Math.min(1.0, Math.max(0.0, confidence));

	}

	/**
	 * (ball <x> <y> <confidence>)
	 *
	 public SoccerBall(String str) {
	 try {
	 if(!str.startsWith("(") || !str.endsWith(")"))
	 throw new ParseErrorException("String not beginning/ending with parentesis: "+str);

	 StringTokenizer st = new StringTokenizer(str.substring(1,str.length()-1)," ");
	 String token;
	 if(st.countTokens() != 4)
	 throw new ParseErrorException("Wrong number of arguments/invalid string: "+str);

	 token = st.nextToken();
	 if(!token.equals("ball"))
	 throw new ParseErrorException("Not a valid player: "+str);

	 x = Double.parseDouble(st.nextToken());
	 y = Double.parseDouble(st.nextToken());
	 confidence = Double.parseDouble(st.nextToken());

	 if(confidence < 0.0 || confidence > 1.0)
	 throw new ParseErrorException("Too large/small value(s) in <confidence>:"+str);
	 } catch(NumberFormatException nfe) {
	 throw new ParseErrorException(nfe.toString());
	 }
	 } */

	public void draw(Graphics2D g2, double size) {
	  int alpha = Math.min((int)(confidence*255+64),255);
	    g2.setColor(new Color(255,255,255,alpha));
	    g2.fill(new Ellipse2D.Double(x-size/2, y-size/2, size, size));
	}
    } // end class SoccerBall

    public static class SoccerPlayer {

	private double x;
	private double y;
	private double confidence;
	private int team; // 0=unknown, 1=our, 2=their
	private double bodydir; //angles
	private int no; // 0=unknown, 1..11=known
	private boolean me;

	private Font font;

	public SoccerPlayer(double _x, double _y, double _confidence,
			    int _team, double _bodydir, int _no, boolean _me)
        {
	  x = _x;
	  y = _y;
	  confidence = _confidence;
	  team = _team;
	  bodydir = _bodydir;
	  no = _no;
	  me = _me;

	  // Adjust confidence to appropriate levels, because of fixed
	  // point roundoff errors
	  confidence = Math.min(1.0, Math.max(0.0, confidence));

	    // fix angle to angles used in the real world (outside rs)
	    if(bodydir < 0)
	      bodydir = -bodydir;
	    else
	      bodydir = 360 - bodydir;

	    // fix font stuff
	    font = new Font("Monospaced", Font.BOLD, 2);

	}

	/**
	 * (<player> <x> <y> <team> <no> <bodydir> <confidence>)
	 * <player> = Player | player
	 * <x>,<y>,<no>,<bodydir>,<confidence> like standard int/floats.
	 * <team> = unknown | our | their
	 *
	 public SoccerPlayer(String str) {
	 try {
	 if(!str.startsWith("(") || !str.endsWith(")"))
	 throw new ParseErrorException("String not beginning/ending with parentesis: "+str);

	 StringTokenizer st = new StringTokenizer(str.substring(1,str.length()-1)," ");
	 String token;
	 if(st.countTokens() != 7)
	 throw new ParseErrorException("Wrong number of arguments/invalid string: "+str);

	 token = st.nextToken();
	 if(token.equals("Player"))
	 me = true;
	 else if(token.equals("player"))
	 me = false;
	 else
	 throw new ParseErrorException("Not a valid player: "+str);

	 x = Double.parseDouble(st.nextToken());
	 y = Double.parseDouble(st.nextToken());

	 token = st.nextToken();
	 if(token.equals("unknown"))
	 team = 0;
	 else if(token.equals("our"))
	 team = 1;
	 else if(token.equals("their"))
	 team = 2;
	 else
	 throw new ParseErrorException("Invalid team: "+token+"in string: "+str);

	 no = Integer.parseInt(st.nextToken());
	 bodydir = Double.parseDouble(st.nextToken());
	 confidence = Double.parseDouble(st.nextToken());

	 if((no < 0 || no > 11) ||
	 (bodydir < -180.0 || bodydir > 180.0) ||
	 (confidence < 0.0 || confidence > 1.0))
	 throw new ParseErrorException("Too large/small value(s) in <no>/<bodydir>/<confidence>:"+str);
	 } catch(NumberFormatException nfe) {
	 throw new ParseErrorException(nfe.toString());
	 }
	 }*/

	public void draw(Graphics2D g2, double size) {
	    Color c;
	    int alpha = Math.min((int)(confidence*255+64),255);
	    switch(team) {
	    case 1: // teammate
		c = new Color(255,255,0,alpha);
		break;
	    case 2: // opponent
		c = new Color(255,0,0,alpha);
		break;
	    default:
		c = new Color(255,255,255,alpha);
	    }

	    if(me)
		c = new Color(0,255,0,alpha);

	    g2.setColor(c);
	    g2.fill(new Ellipse2D.Double(x-size/2, y-size/2, size, size));

	    g2.setColor(new Color(0,0,0,(int)(confidence*255)));

	    double start = bodydir + 90;
	    if(start > 360)
		start -= 360;
	    double face_size = size * 0.9;
	    g2.fill(new Arc2D.Double(x-face_size/2, y-face_size/2,
				     face_size, face_size,
				     start, 180, Arc2D.PIE));
	    if(no != 0) {
		g2.setColor(Color.white);
		g2.setFont(font);
		g2.drawString(""+no, (float)(x-face_size/2), (float)(y+face_size/2));
	    }
	}
    } // end class SoccerPlayer


    // Field size
    private final double length = 105.0;
    private final double width = 68.0;
    private final double margin = 5.0;

    // RoboSoc parameters
    private final double penalty_area_length = 16.5;
    private final double penalty_area_width = 40.32;
    private final double goal_area_length = 5.5;
    private final double goal_area_width = 18.32;
    private final double corner_arc_r = 1.0;

    // Drawing parameters
    private final double player_width = 2.8;
    private final double ball_width = 2.0;
    private double scale;

    private Dimension size;
    private AffineTransform transform;

    private Vector field; // Contains the field (Shape-objects)
    private Vector players; // Contains all players (SoccerPlayer-objects)
    private Vector arrows; // Contains all arrows (Arrow-objects)
    private Vector circles; // Contains all circles (Circle-objects)
    private SoccerBall ball;

    private int cycleno;
    private BufferedImage iBuf;

    /**
     */
    public SoccerField(double s) {
	super();

	cycleno = 0;
	players = new Vector();
	arrows = new Vector();
	circles = new Vector();

	setBackground(new Color(0,128,0)); // Grass

	// Create field
	field = new Vector();

	// outline
	field.add(new Rectangle2D.Double(-length/2, -width/2,
					 length, width));
	// Left penalty area
	field.add(new Rectangle2D.Double(-length/2,
					 -penalty_area_width/2,
					 penalty_area_length,
					 penalty_area_width));
	// Right penalty area
	field.add(new Rectangle2D.Double(length/2-penalty_area_length,
					 -penalty_area_width/2,
					 penalty_area_length,
					 penalty_area_width));
	// Left goal area
	field.add(new Rectangle2D.Double(-length/2,
					 -goal_area_width/2,
					 goal_area_length,
					 goal_area_width));
	// Right goal area
	field.add(new Rectangle2D.Double(length/2-goal_area_length,
					 -goal_area_width/2,
					 goal_area_length,
					 goal_area_width));
	// midline
	field.add(new Line2D.Double(0, -width/2, 0, width/2));


	// Fix size and scale
	size = new Dimension();
	setScale(s);
    }

    /**
     */
    public void setScale(double f) {
	scale = f;
	double l = scale*(margin*2+length);
	double w = scale*(margin*2+width);
	size.setSize((int)(l),(int)(w));
	transform = new AffineTransform(scale, 0, 0, scale, l/2, w/2);

	iBuf = new BufferedImage((int)l,(int)w,BufferedImage.TYPE_INT_RGB);
	Graphics2D g2 = (Graphics2D)iBuf.getGraphics();
	g2.setColor(new Color(0,128,0));
	g2.fillRect(0,0,(int)l,(int)w);
	g2.transform(transform);
	g2.setStroke(new BasicStroke((float)(1/scale)));
	g2.setColor(Color.white);
	for(int i=0; i<field.size(); i++)
	  g2.draw((Shape)field.get(i));


    }

    public void setBall(SoccerBall b) {
	ball = b;
    }

    public SoccerBall getBall() {
	return ball;
    }

    /**
     */
    public double getScale() {
	return scale;
    }

    /**
     */
//      public void addPlayer(SoccerPlayer plr) {
//  	players.add(plr);
//      }

    public void addArrows(Vector arrows) {
	synchronized(arrows) {
	    this.arrows.addAll(arrows);
	}
    }

    public void addCircles(Vector circles) {
	synchronized(circles) {
	    this.circles.addAll(circles);
	}
    }

    public void setPlayers(Vector plrs) {
	players = plrs;
    }

    public Vector getPlayers() {
	return players;
    }

    /**
     */
    public void newCycle() {
	synchronized(arrows) {
	    Iterator it = arrows.iterator();
	    while(it.hasNext()) {
		if(!((Arrow)it.next()).newCycle())
		{
		    it.remove();
		}
	    }
	}
	synchronized(circles) {
	    Iterator it = circles.iterator();
	    while(it.hasNext()) {
		if(!((Circle)it.next()).newCycle())
		{
		    it.remove();
		}
	    }
	}
    }


    /**
     */
    public Dimension getPreferredSize() {
	return size;
    }

    /**
     */
    public void paintComponent(Graphics g) {
	Graphics2D g2 = (Graphics2D)g;
	//super.paintComponent(g); // Draw background
	g2.drawImage(iBuf, 0, 0, this);

	g2.transform(transform);
	g2.setStroke(new BasicStroke((float)(1/scale)));
	g2.setColor(Color.white);

	if(ball != null)
	    ball.draw(g2, ball_width);

	synchronized(players) {
	  for(int i=0; i < players.size(); i++) {
	    ((SoccerPlayer)players.get(i)).draw(g2, player_width);
	  }
	}

	synchronized(arrows) {
	    Iterator it = arrows.iterator();
	    while(it.hasNext()) {
		((Arrow)it.next()).draw(g2);
	    }
	}
	synchronized(circles) {
	    Iterator it = circles.iterator();
	    while(it.hasNext()) {
		((Circle)it.next()).draw(g2);
	    }
	}
	try {
	  Thread.currentThread().sleep(100);
	} catch(InterruptedException e) {}
    }
}





